@@ -7,6 +7,16 @@
 #		nicknames for each status. We don't need backwards compatibility in 
 #		custom status system because previous versions don't store them 
 
+#a custom state is stored in the array StatesList
+#each state as an idx (an integer) is saved as argument in
+#StatesList($idx) which is a list of the following elements : (in order)
+# - the name of this state
+# - the custom nick
+# - the number describing the status (0 for Online ....)
+# - the number of lines of the automessage
+# - the automessage itself
+# - the personal message (aka psm)
+
 #///////////////////////////////////////////////////////////////////////////////
 # LoadStateList ()
 # Loads the list of states from the file in the HOME dir
@@ -33,9 +43,6 @@ proc LoadStateList {} {
 # Saves the list of states to the file in the HOME dir
 proc SaveStateList {} {
 	global HOME tcl_platform 
-    if {![file exists [file join ${HOME} states.xml]]} {
-    	return
-    }
 	if {$tcl_platform(platform) == "unix"} {
 		set file_id [open "[file join ${HOME} states.xml]" w 00600]
 	} else {
@@ -46,12 +53,13 @@ proc SaveStateList {} {
 	set idx 0
 	while { $idx <= [expr {[StateList size] - 1}] } {
 		set tmp [StateList get $idx]
-		set tmp0 [::sxml::xmlreplace [lindex $tmp 0]]
-		set tmp1 [::sxml::xmlreplace [lindex $tmp 1]]
-		set tmp2 [::sxml::xmlreplace [lindex $tmp 2]]
-		set tmp4 [::sxml::xmlreplace [lindex $tmp 4]]
-		puts $file_id "   <newstate>\n      <name>$tmp0</name>\n      <nick>$tmp1</nick>"
-		puts $file_id "      <state>$tmp2</state>\n      <message>$tmp4</message>\n   </newstate>\n"
+		set name [::sxml::xmlreplace [lindex $tmp 0]]
+		set nick [::sxml::xmlreplace [lindex $tmp 1]]
+		set state [::sxml::xmlreplace [lindex $tmp 2]]
+		set msg [::sxml::xmlreplace [lindex $tmp 4]]
+		set psm [::sxml::xmlreplace [lindex $tmp 5]]
+		puts $file_id "   <newstate>\n      <name>$name</name>\n      <nick>$nick</nick>"
+		puts $file_id "      <state>$state</state>\n      <message>$msg</message>\n      <psm>$psm</psm>\n   </newstate>\n"
 		incr idx 1
 	}
 	puts $file_id "</states>"
@@ -63,7 +71,7 @@ proc SaveStateList {} {
 # StateList (action argument)
 # Controls information for list of states
 # action can be :
-#	add : add a new state to the list (have to give a list with 3 elements)
+#	add : add a new state to the list (have to give a list with 6 elements)
 #	promote : makes state given by argument (is an index) the last used (for aging)
 #	get : Returns the state given by argument (is an index), returns 0 if dosen't exist
 #	unset : Removes state given by argument (is an index)
@@ -107,7 +115,7 @@ proc StateList { action { argument "" } {argument2 ""} } {
 
 		show {
 			for {set idx 0} {$idx < [array size StatesList]} {incr idx} {
-				puts stdout "$idx : $StatesList($idx)\n"
+				status_log "$idx : $StatesList($idx)\n"
 			}
 			}
 		clear {
@@ -118,6 +126,13 @@ proc StateList { action { argument "" } {argument2 ""} } {
 	}
 }
 
+#///////////////////////////////////////////////////////////////////////////////
+# AddStatesToList (path)
+proc AddStatesToList { path } {
+	for {set idx 0} {$idx < [StateList size] } { incr idx } {
+		$path list insert end "** [lindex [StateList get $idx] 0] **"
+	}
+}
 #///////////////////////////////////////////////////////////////////////////////
 # CreateStatesMenu (path)
 # Creates the menu that will be added under the default states
@@ -178,40 +193,41 @@ proc CreateStatesMenu { path } {
 #		if { [::config::getKey docking] && [winfo exists $iconmenu.otherstates] } {
 #			$iconmenu add cascade -label "   [trans morepersonal]" -menu $iconmenu.otherstates -state disabled
 #			$iconmenu add separator
-#			$iconmenu add command -label "[trans close]" -command "close_cleanup;exit"
+#			$iconmenu add command -label "[trans close]" -command "exit"
 #		}
 		$path add cascade -label "[trans morepersonal]" -menu $path.otherstates
-		$path add separator
-		$path add command -label "[trans newstate]..." -command "EditNewState 0"
+	}
+
+
+	$path add separator
+	$path add command -label "[trans newstate]" -command "EditNewState 0"
+	if { [StateList size] != 0 } {
 		$path add cascade -label "[trans editcustomstate]" -menu $path.editstates
 		$path add cascade -label "[trans deletecustomstate]" -menu $path.deletestates
-	} else {
-		$path add separator
-		$path add command -label "[trans newstate]" -command "EditNewState 0"
-		if { [StateList size] != 0 } {
-			$path add cascade -label "[trans editcustomstate]" -menu $path.editstates
-			$path add cascade -label "[trans deletecustomstate]" -menu $path.deletestates
-		}
-#		if { [::config::getKey docking] && [winfo exists $iconmenu] } {
-#			$iconmenu add separator
-#			$iconmenu add command -label "[trans close]" -command "close_cleanup;exit"
-#		}
 	}
+	#		if { [::config::getKey docking] && [winfo exists $iconmenu] } {
+	#			$iconmenu add separator
+	#			$iconmenu add command -label "[trans close]" -command "exit"
+	#		}
+	
 	$path add separator
 	$path add command -label "[trans changenick]..." -command cmsn_change_name
 	
 	$path add command -label "[trans changedisplaypic]..." -command pictureBrowser 
-
+	
+	$path add command -label "[trans editmyprofile]..." -command "::hotmail::hotmail_profile"
+	
 	$path add command -label "[trans cfgalarmall]..." -command "::alarms::configDialog all"
-#	statusicon_proc [MSN::myStatusIs]
-
+	#	statusicon_proc [MSN::myStatusIs]
+		
 	if { [::config::getKey use_dock] && [winfo exists $iconmenu.imstatus] && $path != "$iconmenu.imstatus" } {
 		CreateStatesMenu $iconmenu.imstatus
 	}
 	if { [::config::getKey use_dock] && [winfo exists $iconmenu.imstatus] && $path == "$iconmenu.imstatus" } {
-		$path delete [expr "[$path index end] - 3"] end
+		$path delete [expr {[$path index end] - 3}] end
 	}
 }
+	
 
 #///////////////////////////////////////////////////////////////////////////////
 # ChCustomState ( idx )
@@ -219,7 +235,7 @@ proc CreateStatesMenu { path } {
 # idx indicates the index of the personal state in the StateList, 
 # otherwise it indicates a normal state change (AWY, BSY, etc)
 proc ChCustomState { idx } {
-	global HOME automessage automsgsent original_nick
+	global HOME automessage automsgsent original_nick original_psm
 	set automessage "-1"
 	set redraw 0
 	if { [string is digit $idx] == 1 } {
@@ -227,6 +243,9 @@ proc ChCustomState { idx } {
 			if {![info exists original_nick] && [::config::getKey storename]} {
 				set original_nick [::abook::getPersonal MFN]
 			}
+			if {![info exists original_psm] && [::config::getKey storename]} {
+				set original_psm [::abook::getPersonal PSM]
+			}
 			#set new_state [lindex [lindex $list_states [lindex [StateList get $idx] 2]] 0]
 			set new_state [::MSN::numberToState [lindex [StateList get $idx] 2]]
 			if { $new_state == [::MSN::myStatusIs] } {
@@ -234,31 +253,57 @@ proc ChCustomState { idx } {
 			}
 			set automessage [StateList get $idx]
 			set newname "[lindex [StateList get $idx] 1]"
+			set newpsm "[lindex [StateList get $idx] 5]"
+			status_log [StateList get $idx]
 			if { $newname != "" } {
-					catch {
-						set nickcache [open [file join ${HOME} "nick.cache"] w]
-						fconfigure $nickcache -encoding utf-8
-						puts $nickcache $original_nick
-						puts $nickcache $newname
-						puts $nickcache [::abook::getPersonal login]
-						close $nickcache
-					}
+				catch {
+					set nickcache [open [file join ${HOME} "nick.cache"] w]
+					fconfigure $nickcache -encoding utf-8
+					puts $nickcache $original_nick
+					puts $nickcache $newname
+					puts $nickcache [::abook::getPersonal login]
+					close $nickcache
+				}
+				
 				set newname [string map { "\\" "\\\\" "\$" "\\\$" } $newname]
 				set newname [string map { "\\\$nick" "\${original_nick}" } $newname]
 				set newname [subst -nocommands $newname]
 				::MSN::changeName [::config::getKey login] $newname
 				StateList promote $idx
 			}
+			if { $newpsm != "" } {
+                                catch {
+                                        set psmcache [open [file join ${HOME} "psm.cache"] w]
+                                        fconfigure $psmcache -encoding utf-8
+                                        puts $psmcache $original_psm
+                                        puts $psmcache $newpsm
+                                        puts $psmcache [::abook::getPersonal login]
+                                        close $psmcache
+                                }
+
+                                set newpsm [string map { "\\" "\\\\" "\$" "\\\$" } $newpsm]
+                                set newpsm [string map { "\\\$psm" "\${original_psm}" } $newpsm]
+                                set newpsm [subst -nocommands $newpsm]
+                                ::MSN::changePSM $newpsm
+                        }
 		}
 	} else {
 		set automessage "-1"
 		if { $idx == [::MSN::myStatusIs]} {
 			set redraw 1
 		}
-		if {[info exists original_nick] && [::config::getKey storename]} {
-			::MSN::changeName [::config::getKey login] $original_nick
-			unset original_nick
-			catch { file delete [file join ${HOME} "nick.cache"] }
+		if {[::config::getKey storename]} {
+			if { [info exists original_nick] } {
+				::MSN::changeName [::config::getKey login] $original_nick
+				unset original_nick
+				catch { file delete [file join ${HOME} "nick.cache"] }
+			}
+                        if { [info exists original_psm] } {
+                                ::MSN::changePSM $original_psm
+                                unset original_psm
+                                catch { file delete [file join ${HOME} "psm.cache"] } 
+                        }
+
 		}
 		set new_state $idx
 	}
@@ -280,7 +325,7 @@ proc ChCustomState { idx } {
 		unset automsgsent
 	}
 	if { $redraw == 1 } {
-		cmsn_draw_online
+		cmsn_draw_online 0 1
 	}
 }
 
@@ -291,7 +336,8 @@ proc ChCustomState { idx } {
 # mode is 1 for adding a temporary state
 # mode is 2 for editing an old state, need to give idx of state to edit
 proc EditNewState { mode { idx "" } } {
-	global stemp chstate
+	global chstate
+	variable stemp
 	if { $mode == 2 } {
 		if { $idx != "" } {
 			if { [StateList get $idx] == 0 } {
@@ -322,13 +368,13 @@ proc EditNewState { mode { idx "" } } {
 	}
 	#ShowTransient .editstate
 
-	set lfname [LabelFrame:create .editstate.lfname -text [trans stateinfo] -font splainf]
-	pack $lfname -anchor n -side top -expand 1 -fill x
+	set lfname [labelframe .editstate.lfname -text [trans stateinfo] -font splainf]
+	#pack $lfname -anchor n -side top -expand 1 -fill x
     
 	frame .editstate.1 -class Degt
 	label .editstate.1.away -image [::skin::loadPixmap prefaway]
 	pack .editstate.1.away -side left -anchor nw
-    
+
 	if { $mode == 0 || $mode == 1 } {
 		label .editstate.1.laway -text [trans statenewtext] -padx 10 -justify left -font splainf
 	} else { 
@@ -345,30 +391,47 @@ proc EditNewState { mode { idx "" } } {
 	$lfname.nickhelp.menu add command -label [trans nick] -command "$lfname.enick insert insert \\\$nick"
 	$lfname.nickhelp.menu add separator
 	$lfname.nickhelp.menu add command -label [trans delete] -command "$lfname.enick delete 0 end"
-	
+	label $lfname.lpsm -text "[trans statepsm] :" -font splainf
+        entry $lfname.epsm -bg #FFFFFF -font splainf  -width 40
+        menubutton $lfname.psmhelp -font sboldf -text "<-" -menu $lfname.psmhelp.menu
+        menu $lfname.psmhelp.menu -tearoff 0
+        $lfname.psmhelp.menu add command -label [trans psm] -command "$lfname.epsm insert insert \\\$psm"
+        $lfname.psmhelp.menu add separator
+        $lfname.psmhelp.menu add command -label [trans delete] -command "$lfname.epsm delete 0 end"
+
 	label $lfname.lstate -text "[trans state] :" -font splainf
 	combobox::combobox $lfname.statebox -editable false -highlightthickness 0 -width 37 -bg #FFFFFF -font splainf -command ""
 	label $lfname.lmsg -text "[trans stateautomsg] :" -font splainf
 	text $lfname.emsg -background white -borderwidth 2 -relief ridge -width 40 -height 5 -font splainf
-	pack .editstate.1 -expand 1 -fill both -side top -pady 15
+	set msgcopypastemenu [CreateCopyPasteMenu $lfname.emsg]
+	bind $lfname.emsg <Button3-ButtonRelease> "tk_popup $msgcopypastemenu %X %Y"
+	pack .editstate.1 -expand false -fill x -side top -pady 15
 	pack .editstate.lfname -expand 1 -fill both -side top
 	grid $lfname.ldesc -row 1 -column 1 -sticky w -pady 5 -padx 5
 	grid $lfname.edesc -row 1 -column 2 -sticky w -pady 5 -padx 5
 	grid $lfname.lnick -row 2 -column 1 -sticky w -pady 5 -padx 5
 	grid $lfname.enick -row 2 -column 2 -sticky w -pady 5 -padx 5
 	grid $lfname.nickhelp -row 2 -column 3 -sticky w -pady 5 -padx 5
-	grid $lfname.lstate -row 3 -column 1 -sticky w -pady 5 -padx 5
-	grid $lfname.statebox -row 3 -column 2 -sticky w -pady 5 -padx 5
-	grid $lfname.lmsg -row 4 -column 1 -sticky nw -pady 10 -padx 5
-	grid $lfname.emsg -row 4 -column 2 -sticky w -pady 10 -padx 5
-
-	# Frame for options
+	if { [::config::getKey protocol] == 11 } {
+                grid $lfname.lpsm -row 3 -column 1 -sticky w -pady 5 -padx 5
+                grid $lfname.epsm -row 3 -column 2 -sticky w -pady 5 -padx 5
+                grid $lfname.psmhelp -row 3 -column 3 -sticky w -pady 5 -padx 5
+	        grid $lfname.lstate -row 4 -column 1 -sticky w -pady 5 -padx 5
+		grid $lfname.statebox -row 4 -column 2 -sticky w -pady 5 -padx 5
+		grid $lfname.lmsg -row 5 -column 1 -sticky nw -pady 10 -padx 5
+		grid $lfname.emsg -row 5 -column 2 -sticky w -pady 10 -padx 5
+	} else {
+		grid $lfname.lstate -row 3 -column 1 -sticky w -pady 5 -padx 5
+                grid $lfname.statebox -row 3 -column 2 -sticky w -pady 5 -padx 5
+                grid $lfname.lmsg -row 4 -column 1 -sticky nw -pady 10 -padx 5
+                grid $lfname.emsg -row 4 -column 2 -sticky w -pady 10 -padx 5
+	}
+	
+	#Frame for options
 	frame .editstate.options -class Degt
 	if { $mode != 2 } {
-		if { [info exists stemp] } {
-			unset stemp
-		}
-		checkbutton .editstate.options.stemp -text "[trans temp_state]" -onvalue 1 -offvalue 0 -font sboldf
+		set stemp 0
+		checkbutton .editstate.options.stemp -text "[trans temp_state]" -onvalue 1 -offvalue 0 -font sboldf -variable stemp
 		pack .editstate.options.stemp -anchor w -side top -padx 10 -pady 0
 	} elseif { $mode == 2 } {
 		set stemp 2
@@ -402,6 +465,7 @@ proc EditNewState { mode { idx "" } } {
 		$lfname.enick insert end [lindex [StateList get $idx] 1]
 		$lfname.statebox select [lindex [StateList get $idx] 2]
 		$lfname.emsg insert end [lindex [StateList get $idx] 4]
+		$lfname.epsm insert end [lindex [StateList get $idx] 5]
 	}
 	#else {
 	#	$lfname.enick insert end [::abook::getPersonal MFN]
@@ -413,6 +477,31 @@ proc EditNewState { mode { idx "" } } {
 	catch {focus .editstate}
 }
 
+#///////////////////////////////////////////////////////////////////////////////
+# CreateCopyPasteMenu ( w )
+# Returns a menu with Cut - Copy - Paste commands
+
+proc CreateCopyPasteMenu { w } {
+	set menu $w.copypaste
+
+        menu $menu -tearoff 0 -type normal
+
+        $menu add command -label [trans cut] \
+                -command "status_log cut\n;tk_textCut $w"
+        $menu add command -label [trans copy] \
+                -command "status_log copy\n;tk_textCopy $w"
+        $menu add command -label [trans paste] \
+                -command "status_log pasteHere\n;pasteHere $w"
+
+        return $menu
+}
+
+#///////////////////////////////////////////////////////////////////////
+proc pasteHere { w } {
+        set contents [ selection get -selection CLIPBOARD ]
+        $w insert insert $contents
+}
+
 #///////////////////////////////////////////////////////////////////////////////
 # ButtonSaveState ( lfname idx )
 # GUI frontend for adding new states or editing old states
@@ -421,9 +510,11 @@ proc EditNewState { mode { idx "" } } {
 # mode is 1 for adding a temporary state
 # mode is 2 for editing an old state, need to give idx of state to edit
 proc ButtonSaveState { lfname { idx "" } } {
-	# Global variables for temp status and changin the new state, from checkbutton on EditNewState
-	global stemp chstate
+	variable stemp
 	set mode $stemp
+
+	# Global variables for temp status and changin the new state, from checkbutton on EditNewState
+	global chstate
 	lappend gui_info [$lfname.edesc get]
 	lappend gui_info [$lfname.enick get]
 	lappend gui_info [$lfname.statebox curselection]
@@ -432,6 +523,7 @@ proc ButtonSaveState { lfname { idx "" } } {
 	set numlines [llength [split $message "\n"]]
 	lappend gui_info $numlines
 	lappend gui_info $message
+	lappend gui_info [$lfname.epsm get]
 	switch $mode {
 		0 {
 			StateList add $gui_info
@@ -456,9 +548,9 @@ proc ButtonSaveState { lfname { idx "" } } {
 	CreateStatesMenu .my_menu
 	if { ($mode == 0 || $mode == 2) && [winfo exists .cfg] } {
 		set cfgname [[.cfg.notebook.nn getframe session].sw.sf getframe]
-		$cfgname.lfname2.f.f.statelist.box delete 0 end
+		$cfgname.lfname2.statelist.box delete 0 end
 		for { set idx 0 } { $idx < [StateList size] } {incr idx } {
-			$cfgname.lfname2.f.f.statelist.box insert end [lindex [StateList get $idx] 0]
+			$cfgname.lfname2.statelist.box insert end [lindex [StateList get $idx] 0]
 		}
 	}
 	SaveStateList
@@ -523,12 +615,16 @@ proc new_state {cstack cdata saved_data cattr saved_attr args} {
 	} else {
 		lappend newstate ""
 	}
-	
+
 	lappend newstate "$sdata(${cstack}:state)"
 	set message "$sdata(${cstack}:message)"
 	set numlines [llength [split $message "\n"]]
 	lappend newstate $numlines
 	lappend newstate $message
+        if { [info exists sdata(${cstack}:psm)] } {
+                lappend newstate "$sdata(${cstack}:psm)"
+        }
+
 	StateList add $newstate
 	return 0
 }
