@@ -10,6 +10,8 @@ namespace eval ::autoupdate {
 			set tlsplatform "linuxppc"
 		} elseif { ($tcl_platform(os) == "Linux") && ($tcl_platform(machine) == "sparc")} {
 			set tlsplatform "linuxsparc"
+		} elseif { ($tcl_platform(os) == "Linux") && ($tcl_platform(machine) == "x86_64")} {
+			set tlsplatform "linuxx86_64"
 		} elseif { $tcl_platform(os) == "Linux" } {
 			set tlsplatform "linuxx86"
 		} elseif { $tcl_platform(os) == "NetBSD"} {
@@ -17,7 +19,7 @@ namespace eval ::autoupdate {
 		} elseif { $tcl_platform(os) == "FreeBSD"} {
 			set tlsplatform "freebsdx86"
 		} elseif { $tcl_platform(os) == "Solaris"} {
-			set tlsplatform "solaris26"
+			set tlsplatform "solaris28"
 		} elseif { $tcl_platform(platform) == "windows"} {
 			set tlsplatform "win32"
 		} elseif { $tcl_platform(os) == "Darwin" } {
@@ -37,12 +39,14 @@ namespace eval ::autoupdate {
 		pack .tlsdown.choose -side top -anchor w -padx 10 -pady 10
 
 		radiobutton .tlsdown.linuxx86 -text "Linux-x86" -variable tlsplatform -value "linuxx86" -font splainf
+		radiobutton .tlsdown.linuxx86_64 -text "Linux-x86_64" -variable tlsplatform -value "linuxx86_64" -font splainf
 		radiobutton .tlsdown.linuxppc -text "Linux-PowerPC" -variable tlsplatform -value "linuxppc" -font splainf
 		radiobutton .tlsdown.linuxsparc -text "Linux-SPARC" -variable tlsplatform -value "linuxsparc" -font splainf
 		radiobutton .tlsdown.netbsdx86 -text "NetBSD-x86" -variable tlsplatform -value "netbsdx86" -font splainf
 		radiobutton .tlsdown.netbsdsparc64 -text "NetBSD-SPARC64" -variable tlsplatform -value "netbsdsparc64" -font splainf
 		radiobutton .tlsdown.freebsdx86 -text "FreeBSD-x86" -variable tlsplatform -value "freebsdx86" -font splainf
 		radiobutton .tlsdown.solaris26 -text "Solaris 2.6 SPARC" -variable tlsplatform -value "solaris26" -font splainf
+		radiobutton .tlsdown.solaris28 -text "Solaris 2.8 SPARC" -variable tlsplatform -value "solaris28" -font splainf
 		radiobutton .tlsdown.win32 -text "Windows" -variable tlsplatform -value "win32" -font splainf
 		radiobutton .tlsdown.mac -text "Mac OS X" -variable tlsplatform -value "mac" -font splainf
 		radiobutton .tlsdown.src -text "[trans sourcecode]" -variable tlsplatform -value "src" -font splainf
@@ -58,12 +62,14 @@ namespace eval ::autoupdate {
 		pack .tlsdown.f.ok -side right -padx 10 -pady 10
 
 		pack .tlsdown.linuxx86 -side top -anchor w -padx 15
+		pack .tlsdown.linuxx86_64 -side top -anchor w -padx 15
 		pack .tlsdown.linuxppc -side top -anchor w -padx 15
 		pack .tlsdown.linuxsparc -side top -anchor w -padx 15
 		pack .tlsdown.netbsdx86 -side top -anchor w -padx 15
 		pack .tlsdown.netbsdsparc64 -side top -anchor w -padx 15
 		pack .tlsdown.freebsdx86 -side top -anchor w -padx 15
 		pack .tlsdown.solaris26 -side top -anchor w -padx 15
+		pack .tlsdown.solaris28 -side top -anchor w -padx 15
 		pack .tlsdown.mac -side top -anchor w -padx 15
 		pack .tlsdown.win32 -side top -anchor w -padx 15
 		pack .tlsdown.src -side top -anchor w -padx 15
@@ -83,57 +89,38 @@ namespace eval ::autoupdate {
 	proc downloadTLS {} {
 		global tlsplatform
 
-		set baseurl "http://osdn.dl.sourceforge.net/sourceforge/amsn/tls1.4.1"
-
 		if { $tlsplatform == "nodown" } {
 			::amsn::infoMsg [trans notls]
 			return
-		} else {
-			switch $tlsplatform {
-				"linuxx86" {
-					set downloadurl "$baseurl-linux-x86.tar.gz"
-				}
-				"linuxppc" {
-					set downloadurl "$baseurl-linux-ppc.tar.gz"
-				}
-				"linuxsparc" {
-					set downloadurl "$baseurl-linux-sparc.tar.gz"
-				}
-				"netbsdx86" {
-					set downloadurl "$baseurl-netbsd-x86.tar.gz"
-				}
-				"netbsdsparc64" {
-					set downloadurl "$baseurl-netbsd-sparc64.tar.gz"
-				}
-				"freebsdx86" {
-					set downloadurl "$baseurl-freebsd-x86.tar.gz"
-				}
-				"solaris26" {
-					set downloadurl "$baseurl-solaris26-sparc.tar.gz"
-				}
-				"win32" {
-					set downloadurl "$baseurl.exe"
-				}
-				"mac" {
-					set downloadurl "[string range $baseurl 0 end-3]5.0-mac.tar.gz"
-				}				
-				"src" {
-					set downloadurl "$baseurl-src.tar.gz"
-				}
-				default {
-					set downloadurl "none"
-				}
-			}
+		}
+
+		set downloadurl "http://www.amsn-project.net/download-tls.php?arch=$tlsplatform"
+
+
+		if {[ catch {set tok [::http::geturl $downloadurl -command "::autoupdate::downloadTLS2 $downloadurl"]} res ]} {
+			::autoupdate::errorDownloadingTLS $res
+			catch {::http::cleanup $tok}
+		}
+	}
+
+	proc downloadTLS2 { downloadurl token } {
+		status_log "status: [http::status $token]\n"
+		if { [::http::ncode $token] == 302} {
+
+
+			upvar #0 $token state
+			array set meta $state(meta)
+			set downloadurl $meta(Location)
+			status_log "got referal to $downloadurl"
 
 			set w .tlsprogress
-			
+
 			if {[winfo exists $w]} {
 				destroy $w
 			}
-			
+
 			toplevel $w
 			wm group $w .
-			#wm geometry $w 350x160
 
 			label $w.l -text "[trans downloadingtls $downloadurl]" -font splainf
 			pack $w.l -side top -anchor w -padx 10 -pady 10
@@ -155,12 +142,15 @@ namespace eval ::autoupdate {
 				$w.close configure -command "::http::reset $tok"
 				wm protocol $w WM_DELETE_WINDOW "::http::reset $tok"
 			}
+		} else {
+			::autoupdate::errorDownloadingTLS "No redirect from $downloadurl"
+			catch {::http::cleanup $token}
 		}
 	}
+		
 
 	proc downloadTLSCompleted { downloadurl token } {
-		global files_dir HOME2 tlsplatform
-
+		global HOME2 tlsplatform
 		status_log "status: [http::status $token]\n"
 		if { [::http::status $token] == "reset" } {
 			::http::cleanup $token
@@ -180,7 +170,9 @@ namespace eval ::autoupdate {
 
 		switch $tlsplatform {
 			"solaris26" -
+			"solaris28" -
 			"linuxx86" -
+			"linuxx86_64" -
 			"linuxppc" -
 			"linuxsparc" -
 			"netbsdx86" -
@@ -189,7 +181,7 @@ namespace eval ::autoupdate {
 			"mac" -
 			"win32" {
 				if { [catch {
-					set file_id [open [file join $files_dir $fname] w]
+					set file_id [open [file join [::config::getKey receiveddir] $fname] w]
 					fconfigure $file_id -translation {binary binary} -encoding binary
 					puts -nonewline $file_id [::http::data $token]
 					close $file_id
@@ -198,9 +190,9 @@ namespace eval ::autoupdate {
 					cd [file join $HOME2 plugins]
 
 					if { $tlsplatform == "win32" } {
-						exec [file join $files_dir $fname] "x" "-o" "-y"
+						exec [file join [::config::getKey receiveddir] $fname] "x" "-o" "-y"
 					} else {
-						exec gzip -cd [file join $files_dir $fname] | tar xvf -
+						exec gzip -cd [file join [::config::getKey receiveddir] $fname] | tar xvf -
 					}
 					cd $olddir
 					::amsn::infoMsg "[trans tlsinstcompleted]"
@@ -211,17 +203,17 @@ namespace eval ::autoupdate {
 			}
 			"src" {
 			if { [catch {
-					set file_id [open [file join $files_dir $fname] w]
+					set file_id [open [file join [::config::getKey receiveddir] $fname] w]
 					fconfigure $file_id -translation {binary binary} -encoding binary
 					puts -nonewline $file_id [::http::data $token]
 					close $file_id
-					::amsn::infoMsg "[trans tlsdowncompleted $fname $files_dir [file join $HOME2 plugins]]"
+					::amsn::infoMsg "[trans tlsdowncompleted $fname [::config::getKey receiveddir] [file join $HOME2 plugins]]"
 				} res ] } {
 					::autoupdate::errorDownloadingTLS $res
 				}
 			}
 			default {
-				::amsn::infoMsg "[trans tlsdowncompleted $fname $files_dir [file join $HOME2 plugins]]"
+				::amsn::infoMsg "[trans tlsdowncompleted $fname [::config::getKey receiveddir] [file join $HOME2 plugins]]"
 			}
 		}
 
@@ -300,8 +292,8 @@ namespace eval ::autoupdate {
 		#Create the update window
 		toplevel $w
 		wm title $w "[trans newveravailable $tmp_data]"
-		set changeloglink "http://amsn.sourceforge.net/wiki/tiki-index.php?page=ChangeLog"
-		set homepagelink "http://amsn.sourceforge.net/"
+		set changeloglink "http://www.amsn-project.net/wiki/index.php/ChangeLog"
+		set homepagelink "http://www.amsn-project.net/"
 		#Create the frames
 		frame $w.top
 		frame $w.top.buttons
@@ -441,13 +433,13 @@ namespace eval ::autoupdate {
 	#Define default directory on the three platforms 
 	#(It's ok For Mac OS X, change it on your platform if you feel it's not good)
 	proc get_default_location {} {
-		global files_dir env
+		global env
 		if {![catch {tk windowingsystem} wsystem] && $wsystem == "aqua"} {
 			set namelocation "Desktop"
 			set defaultlocation "[file join $env(HOME) Desktop]"
 		} elseif { $::tcl_platform(platform)=="windows" } {
 			set namelocation "Received files folder"
-			set defaultlocation "$files_dir"
+			set defaultlocation "[::config::getKey receiveddir]"
 		} else { 
 			set namelocation "Home folder"
 			set defaultlocation "[file join $env(HOME)]"
@@ -518,7 +510,7 @@ namespace eval ::autoupdate {
 	package require http
 
 	proc check_web_version { token } {
-		global version weburl
+		global rcversion weburl
 
 		set newer 0
 
@@ -527,16 +519,18 @@ namespace eval ::autoupdate {
 		if { [::http::status $token] == "ok" && [::http::ncode $token] == 200 } {
 			set tmp_data [string map {"\n" "" "\r" ""} $tmp_data]
 			set lastver [split $tmp_data "."]
-			set yourver [split $version "."]
+			set yourver [split $rcversion "."]
 
 			for {set x 0} {$x<[llength "$lastver"]} {incr x} {
 				if {[lindex $lastver $x] > [lindex $yourver $x]} {
 					set newer 1
 					break
+				} elseif {[lindex $lastver $x] < [lindex $yourver $x]} {
+					break
 				}
 			}
 
-			catch {status_log "check_web_ver: Current= $yourver New=$lastver ($tmp_data)\n"}
+			catch {status_log "check_web_ver: Current= $rcversion New=$tmp_data\n"}
 			#Time in second when the user clicked to not have an alert before 3 days
 			set weekdate [::config::getKey weekdate]
 			#Actual time in seconds
@@ -642,9 +636,9 @@ namespace eval ::autoupdate {
 		}
 
 		::lang::UpdatedLang
-		::plugins::UpdatedPlugins
+		set updatedplugins [::plugins::UpdatedPlugins]
 
-		if { ($::lang::UpdatedLang == "") && ($::plugins::UpdatedPlugins == "") } {
+		if { ($::lang::UpdatedLang == "") && ($updatedplugins == 0) } {
 			::autoupdate::UpdateLangPlugin_close
 			return 0
 		}
@@ -686,16 +680,17 @@ namespace eval ::autoupdate {
 		}
 
 		#Plugin label
-		if {$::plugins::UpdatedPlugins != ""} {
+		if {$updatedplugins == 1} {
 			label $frame.plugintext -text "[trans pluginselector]" -font sboldf
 			pack configure $frame.plugintext -side top -fill x
 		}
 
 		#Checkbox for each plugin
-		foreach plugin $::plugins::UpdatedPlugins {
-			set name [lindex $plugin 6]
-			checkbutton $frame.plugin$name -onvalue 1 -offvalue 0 -text " $name" -variable ::autoupdate::plugin($name) -anchor w
-			pack configure $frame.plugin$name -side top -fill x
+		foreach plugin [::plugins::getPlugins] {
+			if { [::plugins::getInfo $plugin updated] == 1 } {
+				checkbutton $frame.plugin$plugin -onvalue 1 -offvalue 0 -text " $plugin" -variable ::plugins::plugins(${plugin}_updated_selected) -anchor w
+				pack configure $frame.plugin$plugin -side top -fill x
+			}
 		}
 		
 		# Create a frame that will contain the progress of the update
@@ -747,12 +742,9 @@ namespace eval ::autoupdate {
 			::lang::UpdateLang $langcodes
 		}
 
-		if { [info exists ::plugins::UpdatedPlugins] && $::plugins::UpdatedPlugins != ""} {
-			foreach plugin $::plugins::UpdatedPlugins {
-				set name [lindex $plugin 6]
-				if { [::autoupdate::ReadPluginSelected $name] == 1 } {
-					::plugins::UpdatePlugin $plugin
-				}
+		foreach plugin [::plugins::getPlugins] {
+			if { [::plugins::getInfo $plugin updated] == 1 && [::plugins::getInfo $plugin updated_selected] == 1 } {
+				::plugins::UpdatePlugin $plugin
 			}
 		}
 		
@@ -770,14 +762,11 @@ namespace eval ::autoupdate {
 			destroy ".updatelangplugin"
 		}
 	
-		foreach plugin [::plugins::findplugins] {
-			set namespace [lindex $plugin 6]
-			set file [file join $HOME2 $namespace.xml]
-			file delete $file
+		foreach plugin [::plugins::getPlugins] {
+			file delete [file join $HOME2 $plugin.xml]
 		}
 		
 		unset -nocomplain ::lang::UpdatedLang
-		unset -nocomplain ::plugins::UpdatedPlugins
 		
 	}
 
@@ -791,9 +780,10 @@ namespace eval ::autoupdate {
 			set ::autoupdate::lang($langcode) 1
 		}
 
-		foreach plugin $::plugins::UpdatedPlugins {
-			set name [lindex $plugin 6]
-			set ::autoupdate::plugin($name) 1
+		foreach plugin [::plugins::getPlugins] {
+			if { [::plugins::getInfo $plugin updated] == 1 } {
+				set ::plugins::plugins(${plugin}_updated_selected) 1
+			}
 		}
 		
 	}
@@ -808,9 +798,10 @@ namespace eval ::autoupdate {
 			set ::autoupdate::lang($langcode) 0
 		}
 
-		foreach plugin $::plugins::UpdatedPlugins {
-			set name [lindex $plugin 6]
-			set ::autoupdate::plugin($name) 0
+		foreach plugin [::plugins::getPlugins] {
+			if { [::plugins::getInfo $plugin updated] == 1 } {
+				set ::plugins::plugins(${plugin}_updated_selected) 0
+			}
 		}
 		
 	}
@@ -826,14 +817,4 @@ namespace eval ::autoupdate {
 	}
 
 
-	#///////////////////////////////////////////////////////////////////////
-	proc ReadPluginSelected { plugin } {
-
-		set plugins [array get ::autoupdate::plugin]
-		set id [expr {[lsearch $plugins $plugin] + 1}]
-		return [lindex $plugins $id]
-
-	}
-
-
 }
