@@ -1,13 +1,12 @@
 #
-# $Id: config.tcl,v 1.305 2005/11/20 19:19:53 germinator2000 Exp $
+# $Id: config.tcl 7206 2006-09-06 17:08:18Z kakaroto $
 #
 
 namespace eval ::config {
 
 	proc configDefaults {} {
-		global tcl_platform password auto_path advanced_options
+		global tcl_platform password auto_path advanced_options custom_emotions
 
-		::config::setKey protocol "9"		;# Which MSN Protocol do you prefer to use: 9
 		::config::setKey nossl 0			;#Disable the use of SSL, so it doesn't requite TLS package: 0|1
 
 		::config::setKey login ""			;# These are defaults for users without
@@ -42,9 +41,33 @@ namespace eval ::config {
 		::config::setKey dock 1				;#Docking type
 								;#Changed later for windows to 4
 
+		::config::setKey showmailicon 1
+		::config::setKey show_contactdps_in_cl 0		
+
+		::config::setKey autoresizedp 0
+
+		::config::setKey lowrescam 0
+
+		::config::setKey playbackspeed 100
+
+		#Dir for received files
+		if {![catch {tk windowingsystem} wsystem] && $wsystem == "aqua"} {
+		        ::config::setKey receiveddir "[file join $::env(HOME) Desktop/amsn\ received\ files]"
+		} elseif {$tcl_platform(platform) == "unix"} {
+			::config::setKey receiveddir "[file join $::env(HOME) amsn_received]"
+		} elseif {$tcl_platform(platform) == "windows"} {
+			if {[info exists env(USERPROFILE)]} {
+				::config::setKey receiveddir "[file join $::env(USERPROFILE) amsn_received]"
+			} else {
+				::config::setKey receiveddir "[file join [pwd] amsn_received]"
+			}
+		} else {
+			::config::setKey receiveddir "[file join [pwd] amsn_received]"
+		}
+
 		#Some Autodetected options
 		if {$tcl_platform(os) == "Darwin"} {
-			::config::setKey soundcommand "./sndplay \$sound";#Soundplayer for Mac OS 10.3-10.4	
+			::config::setKey soundcommand "./utils/macosx/sndplay \$sound";#Soundplayer for Mac OS 10.3-10.4	
 			::config::setKey browser "open \$url"
 			::config::setKey notifyXoffset 100
 			::config::setKey notifyYoffset 75
@@ -144,7 +167,6 @@ namespace eval ::config {
 		::config::setKey initialftport 6891	;#Initial for to be used when sending file transfers
 		::config::setKey ftautoaccept 0		;#Auto-Accept file transfer request (Off by default)
 		::config::setKey ftautoclose 0		;#Auto-close file transfer windows when finished
-		::config::setKey disable_new_ft_protocol 0	;#Use new FileTransfer protocol (Off by default)
 
 		::config::setKey shownotify 1 			;#Show notify window (in general, see advanced options)
 		::config::setKey clientcaps 1			;#Send x-clientcaps information to others 3rd Messenger
@@ -161,17 +183,23 @@ namespace eval ::config {
 		::config::setKey blockusers 2
 		::config::setKey showblockedgroup 0
 
+		::config::setKey wanttosharecam 0
+
 		::config::setKey emotisounds 1			;#Play sound on certain emoticons
 		::config::setKey animatedsmileys 1		;#Show animated smileys
+		::config::setKey customsmileys 1                ;#Show other people s custom smileys
 
 		#Custom smileys configuration
-		::config::setKey custom_smileys 1
 		::config::setKey logsbydate 1
 		::config::setKey p4c_name ""
 		::config::setKey tabbedchat 2
 		::config::setKey ContainerCloseAction 0
 		::config::setKey showMobileGroup 1
 
+		# Show/hide menu feature
+		::config::setKey showmainmenu -1
+		::config::setKey showcwmenus -1
+
 		#Advanced options, not in preferences window
 		# Create the entry in the list and then, set
 		# the variable at bottom
@@ -204,6 +232,8 @@ namespace eval ::config {
 			[list local removeempty bool removeempty] \
 			[list local tabtitlenick bool tabtitlenick] \
 			[list local showpicnotify bool showpicnotify] \
+			[list local showmailicon bool showmailicon] \
+			[list local autoresizedp bool autoresizedp] \
 			[list title notifyoffset] \
 			[list local notifyXoffset int xoffset] \
 			[list local notifyYoffset int yoffset] \
@@ -223,18 +253,15 @@ namespace eval ::config {
 			[list local autoconnect bool autoconnect autoconnect2] \
 			[list local keepalive bool keepalive natkeepalive]\
 			[list local default_ns_server str notificationserver]\
-			[list local disable_new_ft_protocol bool_inv new_ft_protocol]\
 			[list local lazypicretrieval bool lazypicretrieval]\
 			[list title MSN] \
 			[list local displayp4context bool displayp4context] \
 			[list local p4contextprefix str p4contextprefix] \
 			[list title others] \
 			[list local activeautoupdate bool activeautoupdate] \
-			[list local updateonlycurrentlanguage bool updateonlycurrentlanguage] \
 			[list local allowbadwords bool allowbadwords] \
 			[list local libtls_temp folder TLS tlsexplain] \
 			[list local notifytyping bool notifytyping] \
-			[list local clientcaps bool clientcaps] \
 			[list local lineflushlog bool lineflushlog] \
 			[list local autocheckver bool autocheckver] \
 			[list local storename bool storenickname] \
@@ -257,7 +284,6 @@ namespace eval ::config {
 		::config::setKey default_ns_server "messenger.hotmail.com:1863"
 		::config::setKey start_ns_server "messenger.hotmail.com:1863"
 		::config::setKey activeautoupdate 1		;#Active the auto update
-		::config::setKey updateonlycurrentlanguage 1	;#Only check updates for current language
 		::config::setKey allowbadwords 1		;#Disable censure on nicks
 		::config::setKey enablebanner 1		;#Show or Hide AMSN Banner (By default Show)
 		::config::setKey startontray 0		;#Start amsn on tray icon only (hide contact list)
@@ -277,8 +303,13 @@ namespace eval ::config {
 		::config::setKey p4contextprefix "" ; #Prefix for P4-Context messages
 		::config::setKey notifytimeout 8000 ; #Number of milisecs before the notify will go away
 		::config::setKey globalnick ""		;#The global custom nickname (pattern), disabled by default
+		::config::setKey psmplace 1		;#The place where must be shown the PSM (0: not shown, 1: At the end, 2: In a new line)
 		::config::setKey globaloverride 0		;# Sets whether Global nicknames pattern should override custom nicks, disabled by default
 
+		if { [info exists custom_emotions] } {
+			unset custom_emotions
+		}
+
 		#System options, not intended to be edited (unless you know what you're doing)
 		set password ""
 		::config::setKey withnotebook 0			;#Use notebook tabs in contact list
@@ -444,6 +475,9 @@ namespace eval ::config {
 
 proc save_config {} {
 	global tcl_platform HOME HOME2 version password custom_emotions
+	
+	#saving the plugins
+	::plugins::save_config
 
 	status_log "save_config: saving config for user [::config::getKey login] in $HOME]\n" black
 
@@ -456,6 +490,7 @@ proc save_config {} {
 	} res]} {
 		return 0
 	}
+	fconfigure $file_id -encoding utf-8
 
 	status_log "save_config: saving config_file. Opening of file returned : $res\n"
 	set loginback [::config::getKey login]
@@ -573,12 +608,11 @@ proc load_config {} {
 			#Force the change to not start amsn on tray if someone choosed that in advanced preferences
 			::config::setKey startontray 0
 			# Force the change of the default sound command
-			# For Mac OS X users who used aMSN 0.90 at the beggining
-			set soundmac [string range "[::config::getKey soundcommand]" 1 11]
-				if { $soundmac=="program_dir" } {
-						::config::setKey soundcommand "./sndplay \$sound"
-				}
+			# For Mac OS X users who used aMSN 0.95 at the beggining
+			if {[::config::getKey soundcommand] == "./sndplay" } {
+				::config::setKey soundcommand "./utils/macosx/sndplay \$sound"
 			}
+		}
 	}
 	#Force to change the path to the new path of plwav.exe
 	if {$tcl_platform(platform) == "windows"} {
@@ -617,6 +651,7 @@ proc load_config {} {
 	}
 
 	::config::setKey clientid 0
+	::config::setKey protocol 11
 	::MSN::setClientCap msnc4
 	::MSN::setClientCap inkgif
 	::MSN::setClientCap multip
@@ -639,6 +674,12 @@ proc load_config {} {
 		resetBanner
 	}
 
+        # Show/hide menus when switching profiles 
+    	if {[catch {tk windowingsystem} wsystem] || $wsystem != "aqua"} {
+	    Showhidemenu 0
+	    ::ChatWindow::ShowHideChatWindowMenus . 0
+	}
+
 	#load Snack when being used
 	if { [::config::getKey usesnack] } {
 		if {![catch {package require snack}]} {
@@ -651,6 +692,8 @@ proc load_config {} {
 	}
 
 	::plugins::LoadPlugins
+
+	create_dir "[::config::getKey receiveddir]"
 }
 
 
@@ -918,12 +961,7 @@ proc LoginList { action age {email ""} {lock ""} } {
 
 		lockexists {
 			set tmp_list [array get LockList]
-			set idx [lsearch $tmp_list "$email"]
-			if { $idx == -1 } {
-				return 0
-			} else {
-				return 1
-			}
+			return [lsearch $tmp_list [LoginList getlock "" "$email"]]
 		}
 
 		size {
@@ -1002,8 +1040,6 @@ proc ConfigChange { window email } {
 					config::setKey login $email
 					load_config
 
-					::config::setKey protocol 9
-
 					LoginList add 0 $email
 					set log_dir "[file join ${HOME} logs]"
 					set webcam_dir "[file join ${HOME} webcam]"
@@ -1020,16 +1056,12 @@ proc ConfigChange { window email } {
 		}
 
 		if { [winfo exists .login] } {
-			.login.main.f.f.passentry2 delete 0 end
+			.login.main.passentry2 delete 0 end
 			if { [info exists password] } {
-				.login.main.f.f.passentry2 insert 0 $password
-			}
-			if {[string is digit -strict "[::config::getKey connectas]"]} {
-				.login.main.f.f.statelist select "[::config::getKey connectas]"
-			} else {
-				.login.main.f.f.statelist select "0"
-				status_log "Variable connectas is not digital [::config::getKey connectas]\n" red
+				.login.main.passentry2 insert 0 $password
 			}
+			.login.main.statelist select [get_state_list_idx [::config::getKey connectas]]
+		
 		}
 	}
 }
@@ -1046,7 +1078,7 @@ proc SwitchProfileMode { value } {
 		if { $HOME == $HOME2 } {
 			for { set idx 0 } { $idx <= [LoginList size 0] } { incr idx } {
 				if { [CheckLock [LoginList get $idx]] != -1 } {
-					set window .login.main.f.f.box
+					set window .login.main.box
 					set cb [$window list get 0 [LoginList size 0]]
 					set index [lsearch $cb [LoginList get $idx]]
 					$window select $index
@@ -1059,16 +1091,16 @@ proc SwitchProfileMode { value } {
 				set loginmode 0
 				# Going back to default profile
 				set loginmode 0
-				RefreshLogin .login.main.f.f 1
+				RefreshLogin .login.main 1
 			} elseif { $idx > [LoginList size 0] } {
 				msg_box [trans allprofilesinuse]
 				# Going back to default profile
 				set loginmode 0
-				RefreshLogin .login.main.f.f 1
+				RefreshLogin .login.main 1
 			}
 		# Else we are already in a profile, select that profile in combobox
 		} else {
-			set window .login.main.f.f.box
+			set window .login.main.box
 			set cb [$window list get 0 [LoginList size 0]]
 			set index [lsearch $cb [::config::getKey login]]
 			$window select $index
@@ -1142,13 +1174,14 @@ proc CreateProfile { email } {
 	create_dir $webcam_dir
 
 	# Load default config initially
-	file copy -force [file join $HOME2 config.xml] $newHOMEdir
+	#file copy -force [file join $HOME2 config.xml] $newHOMEdir
 
 	set oldhome $HOME
 	set HOME $newHOMEdir
 
 	::config::setKey login $email
-	load_config
+	::config::configDefaults
+	#load_config
 	save_config
 
 	::config::setKey login $oldlogin
@@ -1164,20 +1197,20 @@ proc CreateProfile { email } {
 	# Redraw combobox with new profile
 	if { [winfo exists .login] } {
 		set loginmode 1
-		RefreshLogin .login.main.f.f 1
-		.login.main.f.f.box list delete 0 end
+		RefreshLogin .login.main 1
+		.login.main.box list delete 0 end
 		set idx 0
 		set tmp_list ""
 		while { [LoginList get $idx] != 0 } {
 			lappend tmp_list [LoginList get $idx]
 			incr idx
 		}
-		eval .login.main.f.f.box list insert end $tmp_list
+		eval .login.main.box list insert end $tmp_list
 		unset idx
 		unset tmp_list
 
 		# Select the new profile in combobox
-		set window .login.main.f.f.box
+		set window .login.main.box
 		set cb [$window list get 0 [LoginList size 0]]
 		set index [lsearch $cb $email]
 		$window select $index
@@ -1248,7 +1281,7 @@ proc CheckLock { email } {
 	set Port [LoginList getlock 0 $email]
 	status_log "CheckLock: LoginList getlock called. Lock=$Port\n" blue
 	if { $Port != 0 } {
-		if { [catch {socket -server phony -myaddr localhost $Port} newlockSock] != 0  } {
+		if { [catch {socket -server phony $Port} newlockSock] != 0  } {
 			status_log "CheckLock Port is already in use: $newlockSock\n" red
 			# port is taken, let's make sure it's a profile lock
 			foreach {local_host} { localhost "[info hostname]" 127.0.0.1 } {
@@ -1258,14 +1291,17 @@ proc CheckLock { email } {
 					fileevent $clientSock readable "lockcltHdl $clientSock"
 					fconfigure $clientSock -buffering line
 					puts $clientSock "AMSN_LOCK_PING"
+					after 5000 [list set response failed]
 					vwait response
-	
-					#set response [gets $clientSock]
+
 					if { $response == "AMSN_LOCK_PONG" } {
 						status_log "CheckLock: Got PONG response\n" green
 						# profile is locked
 						close $clientSock
 						return -1
+					} elseif { $response == "failed" } {
+						status_log "CheckLock: failed to get response from port $Port. Reseting to 0\n" blue
+						LoginList changelock 0 $email 0
 					} else {
 						status_log "CheckLock: another program using port $Port. Reseting to 0\n" blue
 						# other non amsn program is using the lock port, we better reset the lock to 0
@@ -1285,9 +1321,6 @@ proc CheckLock { email } {
 	return 0
 }
 
-proc phony { sock } {
-}
-
 proc lockcltHdl { sock } {
 	global response
 	set response [gets $sock]
@@ -1320,32 +1353,21 @@ proc GetRandomProfilePort { } {
 proc LockProfile { email } {
 	status_log "LockProfile: Locking $email\n" blue
 	global lockSock
-	set trigger 0
-	while { $trigger == 0 } {
+	set tries 0
+	while { $tries < 5 } {
 		set Port [GetRandomProfilePort]
 		status_log "LockProfile: Got random port $Port\n" blue
-		if { [catch {socket -server lockSvrNew -myaddr localhost $Port} newlockSock] == 0  } {
-			# Got one
+		if { [catch {socket -server lockSvrNew $Port} newlockSock] == 0  } {
 			LoginList changelock 0 $email $Port
 			set lockSock $newlockSock
-			set trigger 1
+			break
+		} elseif {$tries >= 5} {
+			::amsn::errorMsg "Unable to get a socket from locahost.\n Check your /etc/hosts file, please."
+			exit 1
 		} else {
-			if { [catch {socket -server lockSvrNew -myaddr [info hostname] $Port} newlockSock] == 0  } {
-				LoginList changelock 0 $email $Port
-				set lockSock $newlockSock
-				set trigger 1
-			} else {
-				if { [catch {socket -server lockSvrNew -myaddr 127.0.0.1 $Port} newlockSock] == 0  } {
-				LoginList changelock 0 $email $Port
-				set lockSock $newlockSock
-				set trigger 1
-				} else {
-					::amsn::errorMsg "Unable to get a socket from locahost.\n Check your /etc/hosts file, please."
-					exit 1
-				}
-			}
+			catch {status_log "unable to create socket on port $Port - $newlockSock"}
+			incr tries
 		}
-
 	}
 #	if { $trigger == 1 } {
 		#vwait events
@@ -1357,12 +1379,12 @@ proc lockSvrNew { sock addr port} {
 	status_log "lockSvrNew: Accepting connection on port $port\n" blue
 
 #	if { $addr == "127.0.0.1" } {
-		fileevent $sock readable "lockSvrHdl $sock"
+		fileevent $sock readable "lockSvrHdl $addr $sock"
 		fconfigure $sock -buffering line
 #	}
 }
 
-proc lockSvrHdl { sock } {
+proc lockSvrHdl { addr sock } {
 	status_log "lockSvrHdl: handling connection\n" blue
 
 	set command [gets $sock]
@@ -1372,9 +1394,11 @@ proc lockSvrHdl { sock } {
 	    close_remote $sock
 	} else {
 		if { $command == "AMSN_LOCK_PING" } {
-			status_log "lockSvrHdl: PING - PONG\n" blue
-
-			catch {puts $sock "AMSN_LOCK_PONG"}
+			# don't allow anyone to ping and know we are an amsn port unless it's coming from the localhost
+			if { $addr == "127.0.0.1" } {
+				status_log "lockSvrHdl: PING - PONG\n" blue
+				catch {puts $sock "AMSN_LOCK_PONG"}
+			}
 		} else {
 		    read_remote $command $sock
 		}
@@ -1391,34 +1415,36 @@ proc lockSvrHdl { sock } {
 #         file after completion, but the registry key may have still been created/removed
 proc WinRegKey { addrem } {
 	if {$::tcl_platform(platform) == "windows"} {
-		if { [catch { set file_id [open "[file join ${::HOME2} addrem.reg]" w]} res]} {
+	    set filename [file join $::env(TEMP) amsn_addrem.reg]
+		if { [catch { set file_id [open "$filename" w]} res]} {
 			msg_box "Failed to create temporary file with error:\n$res"
 			return 0
 		}
 		puts $file_id "REGEDIT4\n"
 		puts $file_id "\[HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\Run\]"
 		if { $addrem == "add" } {
-			set file [string map {"/" "\\\\" " " "\\ "} [info nameofexecutable]]
-			puts $file_id "\"amsn\"=\"\\\"[string map {"/" "\\\\"} [info nameofexecutable]]\\\" \\\"[string map {"/" "\\\\"} [pwd]]\\\\[string map {"/" "\\\\"} [file tail [info script]]]\\\"\"\n"
+		    set amsn_path [file nativename [file normalize [file join [file dirname [info nameofexecutable]] .. amsn.exe]]]
+		    puts $file_id "\"amsn\"=\"\\\"[string map {"\\" "\\\\"} $amsn_path]\\\"\"\n"
 		} else {
-			puts $file_id "\"amsn\"=-"
+		    puts $file_id "\"amsn\"=-"
 		}
 		
 		close $file_id
 
-		if { [catch { exec regedit /s [file join ${::HOME2} addrem.reg]} res]} {
+		if { [catch { exec regedit /s "$filename"} res]} {
 			msg_box "Failed to create/remove the registry key with error:\n$res"
 			return 0
 		}
 		
-		if { [catch { file delete "[file join ${::HOME2} addrem.reg]"} res]} {
+		if { [catch { file delete "$filename"} res]} {
 			msg_box "Failed to delete temporary file with error:\n$res"
 			return 0
 		}
 		return 1
 	}
 	return 0
-}
+} 
+
 #///////////////////////////////////////////////////////////////////////
 
 
@@ -1448,7 +1474,7 @@ if { $initialize_amsn == 1 } {
 	create_dir $HOME/plugins
 	create_dir $HOME/skins
 	#create_dir $log_dir
-	#create_dir $files_dir
+	#create_dir [::config::getKey receiveddir]
 	scan_languages
 	::config::configDefaults
 	::config::loadGlobal
