@@ -53,6 +53,7 @@ namespace eval ::Nudge {
 		::plugins::RegisterEvent Nudge chatmenu itemmenu
 		::plugins::RegisterEvent Nudge right_menu clitemmenu
 		::plugins::RegisterEvent Nudge AllPluginsLoaded add_command
+		::plugins::RegisterEvent Nudge chatwindowbutton blockbutton
 	}
 	
 	########################################
@@ -71,6 +72,10 @@ namespace eval ::Nudge {
 			shakes {10}
 			shaketoo {0}
 			addbutton {1}
+			addblockbutton {0}
+			addclmenuitem {0}
+			limit {1}
+			delay {60}
 		}
 	}
 	
@@ -90,6 +95,10 @@ namespace eval ::Nudge {
 			[list bool "$::Nudge::language(sound_send)" soundnotsend] \
 			[list bool "$::Nudge::language(sound_receive)" soundnotrec] \
 			[list bool "$::Nudge::language(add_button)" addbutton] \
+			[list bool "$::Nudge::language(add_blockbutton)" addblockbutton] \
+			[list bool "$::Nudge::language(add_clmenuitem)" addclmenuitem] \
+			[list bool "$::Nudge::language(limit)" limit] \
+			[list str "\t$::Nudge::language(delay):" delay] \
 		]
 	}
 	########################################
@@ -100,12 +109,12 @@ namespace eval ::Nudge {
 	########################################
 	proc load_nudge_languages {dir} {
 		#Verify if aMSN version is 0.94 or better
-		if {[::Nudge::version_094]} {
+		if { [::Nudge::version_094] } {
 			#If 0.94, use only english keys because 0.94 is not compatible with langfiles in plugins
 			::Nudge::language_array_094
 		} else {
 			#Load lang files
-			set langdir [append dir "/lang"]
+			set langdir [file join $dir "lang"]
 			set lang [::config::getGlobalKey language]
 			load_lang en $langdir
 			load_lang $lang $langdir
@@ -129,9 +138,16 @@ namespace eval ::Nudge {
 			sound_send "Play a sound upon sending a nudge" \
 			sound_receive "Play a sound upon receiving a nudge." \
 			add_button "Add a button to send a nudge in the chatwindow" \
+			add_blockbutton "Add a button to block incoming nudges in the chatwindow" \
+			add_clmenuitem "Add an item to the contactlist popup-menu" \
 			send_nudge "Send nudge" \
 			no_nudge_support "You cannot sent a nudge to your contact because he or she doesn't use a client that supports nudges" \
 			nudge_sent "You have just sent a Nudge" \
+			limit "Activate Nudge receive limitation" \
+			delay "Minimum time between 2 nudge from the same contact (in seconds)" \
+			block_nudge "Block/Unblock Nudges" \
+			no_nudge_support "You cannot sent a nudge to your contact because he or she doesn't use a client that supports nudges" \
+			nudge_sent "You have just sent a Nudge" \
 		]
 	}
 	
@@ -142,11 +158,15 @@ namespace eval ::Nudge {
 	# Compatible with lang files           #
 	########################################
 	proc language_array {} {
-		array set ::Nudge::language [list shake_receive "[trans shake_receive]" shake_send "[trans shake_send]" \
-			shake_nudge "[trans shake_nudge]" popup_nudge "[trans popup_nudge]" notify_send "[trans notify_send]" \
-			notify_receive "[trans notify_receive]" sound_send "[trans sound_send]" sound_receive "[trans sound_receive]" \
-			add_button "[trans add_button]" send_nudge "[trans send_nudge]" no_nudge_support "[trans no_nudge_support]" \
-			nudge_sent "[trans nudge_sent]" \
+		array set ::Nudge::language [list shake_receive "[trans shake_receive]" \
+			shake_send "[trans shake_send]" shake_nudge "[trans shake_nudge]" \
+			popup_nudge "[trans popup_nudge]" notify_send "[trans notify_send]" \
+			notify_receive "[trans notify_receive]" sound_send "[trans sound_send]" \
+			sound_receive "[trans sound_receive]" add_button "[trans add_button]" \
+			add_blockbutton "[trans add_blockbutton]" \
+			add_clmenuitem "[trans add_clmenuitem]" send_nudge "[trans send_nudge]" \
+			no_nudge_support "[trans no_nudge_support]" nudge_sent "[trans nudge_sent]" \
+			limit "[trans limit]" delay "[trans delay]" block_nudge "[trans block_nudge]" \
 		]
 	}
 	
@@ -161,6 +181,19 @@ namespace eval ::Nudge {
 		upvar 2 $args(chatid) chatid
 		upvar 2 $args(nick) nick
 		upvar 2 $args(msg) msg
+		set timestamp [clock seconds]
+		set lastnudge [::abook::getVolatileData $chatid last_nudge]
+		set auth [::abook::getContactData $chatid auth_nudge]	
+		
+		if {$auth == ""} {
+			set auth 1
+		}
+		if {$lastnudge == ""} {
+			set lastnudge 0
+		}
+		
+		set lastnudge [expr $timestamp-$lastnudge]
+
 		#The way to get headers change on 0.95
 		if {[::Nudge::version_094]} {
 			set header "[::MSN::GetHeaderValue $msg Content-Type]"
@@ -169,8 +202,14 @@ namespace eval ::Nudge {
 			set header "[$msg getHeader Content-Type]"
 			set ID "[$msg getField ID]"
 		}
+		
+		#If the user choosed to desactivate nudge receive limitation
+		if {$::Nudge::config(limit) == 0} {
+			set lastnudge [expr $lastnudge+$::Nudge::config(delay)]
+			set auth 1
+		}
 
-		if {$header == "text/x-msnmsgr-datacast" && $ID == "1"} {
+		if {$header == "text/x-msnmsgr-datacast" && $ID == "1" && $lastnudge > $::Nudge::config(delay) && $auth=="1"} {
 			::Nudge::log "Start receiving nudge from <[::abook::getDisplayNick $chatid]>"
 			#If the user choosed to have the nudge notified in the window
 			if { $::Nudge::config(notsentinwin) == 1 } {
@@ -203,6 +242,7 @@ namespace eval ::Nudge {
 				::Nudge::log "Show growl notification"
 			}
 			
+			::abook::setVolatileData $chatid last_nudge $timestamp
 			::Nudge::log "Receiving nudge from <[::abook::getDisplayNick $chatid]> is finished"		
 		
 		}
@@ -217,7 +257,7 @@ namespace eval ::Nudge {
 	###############################################
 	proc notify { nick email } {
 		#Get a shorter nick-name for the notify window
-		set maxw [expr {[::config::getKey notifwidth]-20}]
+		set maxw [expr {[::skin::getKey notifwidth]-20}]
 		set nickname [trunc $nick . $maxw splainf]
 		#Show the notification
 		::amsn::notifyAdd "Nudge\n[trans nudge $nickname]." "::amsn::chatUser $email" "" plugins
@@ -310,7 +350,7 @@ namespace eval ::Nudge {
 			::Nudge::log "Nudge button added the new window: $newvar(window_name)"
 		}
 	}
-
+	
 	################################################
 	# ::Nudge::itemmenu event epvar                #
 	# -------------------------------------------  #
@@ -321,7 +361,7 @@ namespace eval ::Nudge {
 	proc itemmenu { event evpar } {
 		upvar 2 $evpar newvar
 		#Add a separator to the menu
-		$newvar(menu_name).actions add separator
+#		$newvar(menu_name).actions add separator
 		#Add label in the menu
 		$newvar(menu_name).actions add command -label "$::Nudge::language(send_nudge)" \
 		-command "::Nudge::send_via_queue \[::ChatWindow::getCurrentTab $newvar(window_name)\]"
@@ -355,13 +395,21 @@ namespace eval ::Nudge {
 	################################################	
 	proc clitemmenu { event evpar } {
 		upvar 2 $evpar newvar
-		#Add a separator to the menu
-		$newvar(menu_name) add separator
-		
-		#Add label in the menu
-		$newvar(menu_name) add command -label "$::Nudge::language(send_nudge)" \
-		-command "::Nudge::ClSendNudge $newvar(user_login)"
-		::Nudge::log "Create Send Nudge item in right click menu"
+
+		if { $::Nudge::config(addclmenuitem) == 1 } {
+			#Add separator and label in the menu
+			if { [winfo exists ${newvar(menu_name)}.actions] } {
+				${newvar(menu_name)}.actions add separator
+				${newvar(menu_name)}.actions add command -label "$::Nudge::language(send_nudge)" \
+				-command "::Nudge::ClSendNudge $newvar(user_login)"
+			} else {
+				
+
+				$newvar(menu_name) insert [trans viewprofile] command -label "$::Nudge::language(send_nudge)" \
+				-command "::Nudge::ClSendNudge $newvar(user_login)"
+			}
+			::Nudge::log "Create Send Nudge item in right click menu"
+		}
 	}
 
 	################################################  
@@ -418,7 +466,21 @@ namespace eval ::Nudge {
 		set chatid [::ChatWindow::Name $window_name]
 		::Nudge::log "\nStart sending Nudge to <[::abook::getDisplayNick $chatid]>\n"
 		#Check if the user can accept the nudge (MSN 7 protocol needed), if not, stop here.
-			if {![::Nudge::check_clientid $chatid]} {
+		set theysupport 0
+
+		set users [::MSN::usersInChat $chatid]
+	
+		foreach chatid2 $users {
+			if {[::Nudge::check_clientid $chatid2]} {
+				#This is what the official client does...
+				#sends nudge to a multi-convo even when
+				#not everyone supports it
+				set theysupport 1
+				break
+			}
+		}
+
+		if { $theysupport == 0 } {
 				::Nudge::winwrite $chatid \
 				"$::Nudge::language(no_nudge_support)" nudgeoff red
 				::Nudge::log "Can't send a Nudge to <[::abook::getDisplayNick $chatid]> because he doesn't use MSN 7 protocol"
@@ -509,6 +571,10 @@ namespace eval ::Nudge {
 
 		status_log "Clientid is $clientid"
 	
+		if { $clientid == "" } {
+			return 0
+		}
+	
 		foreach bit $supportedclients {
 			status_log "Bitmask is $bit"
 			if {($clientid & $bit) == $bit} {
@@ -575,4 +641,61 @@ namespace eval ::Nudge {
 			::skin::setPixmap nudgebutton nudgebutton.gif
 			::skin::setPixmap nudgebutton_hover nudgebutton_hover.gif
 	}
+
+	################################################
+	# ::Nudge::blockbutton event epvar             #
+	# -------------------------------------------  #
+	# Button to add in the chat window             #
+	# When we click on that button,we block/deblock#
+	# Nudge ability for that contact               #
+	################################################	
+	proc blockbutton { event evpar } {
+		if { $::Nudge::config(addblockbutton) == 1 } {
+			upvar 2 $evpar newvar
+			set nudgebutton $newvar(bottom).nudgeblock
+			set chatid [::ChatWindow::Name $newvar(window_name)]
+			if {[::abook::getContactData $chatid auth_nudge] == "0"} {
+				set nudgeimg "nudgeoff"
+			} else {
+				set nudgeimg "nudge"
+			}
+			#Create the button with an actual Pixmap
+			#Use after 1 to avoid a bug on Mac OS X when we close the chatwindow before the end of the nudge
+			#Keep compatibility with 0.94 for the getColor
+			if {[::Nudge::version_094]} {
+				label $nudgebutton -image [::skin::loadPixmap $nudgeimg] -relief flat -padx 0 \
+				-background [::skin::getColor background2] -highlightthickness 0 -borderwidth 0 \
+				-highlightbackground [::skin::getColor background2] -activebackground [::skin::getColor background2]\
+			} else {
+				label $nudgebutton -image [::skin::loadPixmap $nudgeimg] -relief flat -padx 0 \
+				-background [::skin::getKey buttonbarbg] -highlightthickness 0 -borderwidth 0 \
+				-highlightbackground [::skin::getKey buttonbarbg] -activebackground [::skin::getKey buttonbarbg]\
+			}
+			bind $nudgebutton <<Button1>> "after 1 ::Nudge::blocknudge $chatid $nudgebutton"
+			
+			#Define baloon info
+			set_balloon $nudgebutton "$::Nudge::language(block_nudge)"
+		
+			#Pack the button in the right area
+			pack $nudgebutton -side right
+			::Nudge::log "Nudge block button added the new window: $newvar(window_name)"
+		}
+	}
+
+	##############################################
+	# ::Nudge::blocknudge                        #
+	# -------------------------------------------#
+	# Set/Unset nudge authorization for a contact#
+	# Change picture in the chat window according#
+	# to the state.                              #
+	##############################################
+	proc blocknudge {chatid button} {
+			if {[::abook::getContactData $chatid auth_nudge] == "1"} {
+				::abook::setContactData $chatid auth_nudge 0
+				$button configure -image [::skin::loadPixmap nudgeoff]
+			} else {
+				::abook::setContactData $chatid auth_nudge 1
+				$button configure -image [::skin::loadPixmap nudge]
+			}			
+		}
 }
