@@ -30,11 +30,11 @@ namespace eval dkfprogress {
 
 	frame $w.l -borderwidth 0 -background $undoneBackground
 	label $w.l.l -textvariable $varname -borderwidth 0 \
-		-foreground $undoneForeground -background $undoneBackground
+		-foreground black -background $undoneBackground
 	$w.l configure -height [expr {int([winfo reqheight $w.l.l]+2)}]
 	frame $w.l.fill -background $doneBackground
 	label $w.l.fill.l -textvariable $varname -borderwidth 0 \
-		-foreground $doneForeground -background $doneBackground
+		-foreground black -background $doneBackground
 
 	bind $w.l <Configure> [namespace code [list ProgressConf $w "%w"]]
 
@@ -53,15 +53,27 @@ namespace eval dkfprogress {
 
     proc SetProgress {win value {range 100}} {
 		if {[winfo exists $win]} {
-			set progress [expr {int(100*$value)/int($range)}]
+			set progress [expr {int(double($value)/ (double($range)/100.0))}]
 			set relwidth [expr {double($value)/double($range)}]
 			variable progressPercent
 			place conf $win.l.fill -relwidth $relwidth
 			set progressPercent($win) "${progress}%"
+			
+			if {[expr $relwidth < 0.5]} {
+				set R e1
+				binary scan [binary format i [expr {int(2*$relwidth*225)}]] H2 G
+			} else {
+				set G e1
+				binary scan [binary format i [expr {int(2*(1.0 - $relwidth)*225)}]] H2 R
+			}
+			set B 00
+			$win.l.fill configure -background \#${R}${G}${B}
+			$win.l.fill.l configure -background \#${R}${G}${B}
 		}
     }
 }
 
+
 if { $initialize_amsn == 1 } {
     namespace import dkfprogress::Progress dkfprogress::SetProgress
 }
