@@ -5,7 +5,7 @@
 #	converted to namespace and improved by D. Emilio Grimaldo T.
 #	SOCKS5 support (integration) is experimental!!!
 #
-# RCS: @(#) $Id: proxy.tcl,v 1.51 2005/11/22 20:22:16 lephilousophe Exp $
+# RCS: @(#) $Id: proxy.tcl 7327 2006-10-06 07:22:18Z kakaroto $
 
 package provide Proxy 0.1
 package require http
@@ -17,7 +17,7 @@ source socks.tcl	;# SOCKS5 proxy support
 #never the other proxy objects directly
 
 proc globalGotNexusReply { proxy token {total 0} {current 0} } {
-	if {[info procs $proxy] != ""} {
+	if {![catch {$proxy cget -name}]} {
 		$proxy GotNexusReply $token $total $current
 	} else {
 		::http::cleanup $token
@@ -25,7 +25,7 @@ proc globalGotNexusReply { proxy token {total 0} {current 0} } {
 }
 
 proc globalGotAuthReply { proxy str token } {
-	if {[info procs $proxy] != ""} {
+	if {![catch {$proxy cget -name}]} {
 		$proxy GotAuthReply $str $token
 	} else {
 		::http::cleanup $token
@@ -33,11 +33,66 @@ proc globalGotAuthReply { proxy str token } {
 }
 
 proc globalWrite { proxy name {msg ""} } {
-	if {[info procs $proxy] != ""} {
+	if {![catch {$proxy cget -name}]} {
 		$proxy write $name $msg
 	}
 }
 
+#The only way to get HTTP proxy + SSL to work...
+#http://wiki.tcl.tk/2627
+#helped by patthoyts autoproxy!
+proc secureSocket { args } {
+	set phost [::http::config -proxyhost]
+	set pport [::http::config -proxyport]
+	upvar host thost
+	upvar port tport
+
+	# if a proxy has been configured
+	if {[string length $phost] && [string length $pport]} {
+		#TODO: make async: set socket [socket -async $phost $pport]
+		# create the socket to the proxy
+		set socket [socket -async $phost $pport]
+		fconfigure $socket -buffering line -translation crlf
+		puts $socket "CONNECT $thost:$tport HTTP/1.0"
+        	puts $socket "Host: $thost"
+	        puts $socket "User-Agent: [http::config -useragent]"
+		puts $socket "Content-Length: 0"
+	        puts $socket "Proxy-Connection: Keep-Alive"
+	        puts $socket "Connection: Keep-Alive"
+		puts $socket "Pragma: no-cache"
+		if { [::config::getKey proxyauthenticate] } {
+			set proxy_user [::config::getKey proxyuser]
+			set proxy_pass [::config::getKey proxypass]
+			puts $socket "Proxy-Authorization: Basic [base64::encode ${proxy_user}:${proxy_pass}]"
+		}
+		puts $socket ""
+		#flush $socket
+
+		set reply ""
+		while {[gets $socket r] > 0} {
+			lappend reply $r
+		}
+
+		set result [lindex $reply 0]
+		set code [lindex [split $result { }] 1]
+
+		# be sure there's a valid response code
+		# We use a regexp because of some (or maybe only one) proxy returning "HTTP/1.0 200 .." with two spaces, 
+		# so the split makes the code the 3rd argument not the second, and $code becomes empty. 
+		# refer to http://amsn.sf.net/forums/viewtopic.php?t=1030
+		if {! [regexp {^HTTP/1\.[01] +2[0-9][0-9]} $result]} {
+			return -code error $result
+		}
+
+		# now add tls to the socket and return it
+		fconfigure $socket -blocking 0 -buffering none -translation binary
+		return [::tls::import $socket]
+	}
+
+	# if not proxifying, just create a tls socket directly
+	return [::tls::socket $thost $tport]
+}
+
 ::snit::type Proxy {
 
 	delegate method * to proxy
@@ -148,31 +203,31 @@ proc globalWrite { proxy name {msg ""} } {
 		global tlsinstalled login_passport_url
 
 		#Check if we need to install the TLS module
-		if { $tlsinstalled == 0 && [checking_package_tls] == 0 && [::config::getKey nossl] == 0} {
+		if { $tlsinstalled == 0 && [checking_package_tls] == 0} {
 			::autoupdate::installTLS
 			return -1
 		}
 
 		#If SSL is used, register https:// protocol
-		if { [::config::getKey nossl] == 0 } {
+#		if { [::config::getKey nossl] == 0 } {
 			http::register https 443 ::tls::socket
-		} else  {
-			catch {http::unregister https}
-		}
+#		} else  {
+#			catch {http::unregister https}
+#		}
 
 		#No proxy is used
 		::http::config -proxyhost ""
 
-		if { [::config::getKey nossl] == 1 } {
-			#If we can't use ssl, avoid getting url from nexus
-			set login_passport_url "https://login.passport.com/login2.srf"
-		} else {
+#		if { [::config::getKey nossl] == 1 } {
+#			#If we can't use ssl, avoid getting url from nexus
+#			set login_passport_url "https://login.passport.com/login2.srf"
+#		} else {
 			#Contact nexus to get login url
 			set login_passport_url 0
 			degt_protocol $self
 
 			after 0 "catch {::http::geturl [list https://nexus.passport.com/rdr/pprdr.asp] -timeout 10000 -command {globalGotNexusReply $self}}"
-		}
+#		}
 	}
 
 	method authenticate {str url} {
@@ -181,9 +236,9 @@ proc globalWrite { proxy name {msg ""} } {
 		#if { [::config::getKey nossl] == 1 || ([::config::getKey connectiontype] != "direct" && [::config::getKey connectiontype] != "http") } {
 		#	set url [string map { https:// http:// } $url]
 		#}
-		if { [::config::getKey nossl] == 1 } {
-			set url [string map { https:// http:// } $url]
-		}
+#		if { [::config::getKey nossl] == 1 } {
+#			set url [string map { https:// http:// } $url]
+#		}
 		status_log "::DirectConnection::authenticate: Getting $url\n" blue
 		if { [catch {::http::geturl $url -command "globalGotAuthReply $self [list $str]" -headers $head}] } {
 			eval [ns cget -autherror_handler]
@@ -280,7 +335,7 @@ proc globalWrite { proxy name {msg ""} } {
 #		variable proxy_queued_data
 #		variable proxy_session_id
 #		variable proxy_gateway_ip
-#		variable options(-proxy_writing)
+#		variable options
 
 		#Cancel any previous attemp to write or POLL
 		after cancel "$self PollPOST $name"
@@ -347,7 +402,23 @@ proc globalWrite { proxy name {msg ""} } {
 	}
 
 	method authInit {} {
-		catch {::http::unregister https}
+		#catch {::http::unregister https}
+
+		global tlsinstalled login_passport_url
+
+                #Check if we need to install the TLS module
+                if { $tlsinstalled == 0 && [checking_package_tls] == 0} {
+                        ::autoupdate::installTLS
+                        return -1
+                }
+
+                #If SSL is used, register https:// protocol
+#                if { [::config::getKey nossl] == 0 } {
+#                        http::register https 443 ::tls::socket
+#                } else  {
+#                        catch {http::unregister https}
+#                }
+
 
 		set proxy_host [ns cget -proxy_host]
 		set proxy_port [ns cget -proxy_port]
@@ -358,10 +429,39 @@ proc globalWrite { proxy name {msg ""} } {
 				set proxy_port 8080
 			}
 
-			::http::config -proxyhost $proxy_host -proxyport $proxy_port
+                	if { [::config::getKey proxyauthenticate] } {
+	                        set proxy_user [::config::getKey proxyuser]
+	                        set proxy_pass [::config::getKey proxypass]
+	                }
+	                ::http::config -proxyhost $proxy_host -proxyport $proxy_port
 		}
 
-		set ::login_passport_url "https://login.passport.com/login2.srf"
+		# http://wiki.tcl.tk/2627 :(
+		if { [catch {http::register https 443 secureSocket} res]} {
+			MSN::logout
+			MSN::reconnect "Proxy returned error: $res"
+			return -1
+		}
+
+#		set ::login_passport_url "https://login.passport.com/login2.srf"
+#		        if { [::config::getKey nossl] == 1 } {
+                        #If we can't use ssl, avoid getting url from nexus
+#                        set login_passport_url "https://login.passport.com/login2.srf"
+#                } else {
+                        #Contact nexus to get login url
+                        set login_passport_url 0
+                        degt_protocol $self
+
+			#::http::geturl [list https://nexus.passport.com/rdr/pprdr.asp] -timeout 10000 -command [list globalGotNexusReply $self]
+                        if { [catch {::http::geturl https://nexus.passport.com/rdr/pprdr.asp -timeout 10000 -command [list globalGotNexusReply $self]} res]} {
+				MSN::logout
+				MSN::reconnect "proxy error: $res"
+				return -1
+			}
+
+#                }
+		return 1
+
 	}
 
 	method authenticate {str url} {
@@ -377,13 +477,45 @@ proc globalWrite { proxy name {msg ""} } {
 			lappend head "Proxy-Authorization" "Basic [::base64::encode $proxy_user:$proxy_password]"
 		}
 
-		set url [string map { https:// http:// } $url]
+		#set url [string map { https:// http:// } $url]
 		status_log "::HTTPConnection::authenticate: Getting $url\n" blue
 		::http::geturl $url -command "$self GotAuthReply [list $str]" -headers $head
 #			eval [ns cget -autherror_handler]
 #
 	}
 
+        method GotNexusReply {token {total 0} {current 0}} {
+
+                global login_passport_url
+                if { [::http::status $token] != "ok" || [::http::ncode $token ] != 200 } {
+                        #Nexus connection failed, so let's just set login URL manually
+                        set loginurl "https://login.passport.com/login2.srf"
+                        status_log "gotNexusReply: error in nexus reply, getting url manually\n" red
+                } else {
+                        #We got reply from nexus. Extract login URL
+                        upvar #0 $token state
+
+                        set index [expr {[lsearch $state(meta) "PassportURLs"]+1}]
+                        set values [split [lindex $state(meta) $index] ","]
+                        set index [lsearch $values "DALogin=*"]
+                        set loginurl "https://[string range [lindex $values $index] 8 end]"
+                        status_log "gotNexusReply: loginurl=$loginurl\n" green
+                }
+                ::http::cleanup $token
+
+                #If $login_passport_url == 0, we got login url before authentication took place
+                if { $login_passport_url == 0 } {
+                        #Set loginurl (will be used in authentication), and rest in peace
+                        set login_passport_url $loginurl
+                        status_log "gotNexusReply: finished before authentication took place\n" green
+                } else {
+                        #Authentication is waiting for us to get this url!! Do authentication inmediatly
+                        status_log "gotNexusReply: authentication was waiting for me, so I'll do it\n" green
+                        $self authenticate $login_passport_url $loginurl
+                }
+
+        }
+
 	method GotAuthReply { str token } {
 		if { [::http::status $token] != "ok" } {
 			::http::cleanup $token
@@ -557,7 +689,7 @@ proc globalWrite { proxy name {msg ""} } {
 	}
 
 	method RetryWrite { name } {
-#		variable options(-proxy_writing)
+#		variable options
 		status_log "Retrying write\n" blue
 		catch {fileevent [$name cget -sock] writable ""}
 		if { [catch {puts -nonewline [$name cget -sock] $options(-proxy_writing)} res] } {
@@ -578,7 +710,7 @@ proc globalWrite { proxy name {msg ""} } {
 		variable proxy_session_id
 #		variable proxy_gateway_ip
 		variable proxy_data
-		variable options(-proxy_writing)
+		variable options
 
 		after cancel "$self HTTPPoll $name"
 
@@ -652,16 +784,17 @@ proc globalWrite { proxy name {msg ""} } {
 
 					#degt_protocol "<-Proxy($name) $command" nsrecv
 
-					if {[string range $command 0 2] == "MSG"} {
+					if {[lsearch {MSG NOT PAG IPG UBX GCF} [string range $command 0 2]] != -1} {
 						set recv [split $command]
-						set msg_data [string range $log 0 [expr {[lindex $recv 3]-1}]]
-						set log [string range $log [expr {[lindex $recv 3]}] end]
-
+						set msg_data [string range $log 0 [expr {[lindex $recv end]-1}]]
+						set log [string range $log [expr {[lindex $recv end]}] end]
+						set command [encoding convertfrom utf-8 $command]
 						$name handleCommand $command $msg_data
 						#degt_protocol " Message contents:\n$msg_data" msgcontents
 
 						#sb append $name data $msg_data
 					} else {
+						set command [encoding convertfrom utf-8 $command]
 						$name handleCommand $command
 					}
 
@@ -683,7 +816,7 @@ proc globalWrite { proxy name {msg ""} } {
 #		variable proxy_session_id
 #		variable proxy_gateway_ip
 #		variable proxy_queued_data
-#		variable options(-proxy_writing)
+#		variable options
 
 		if { ![info exists options(-proxy_session_id)]} {
 			return
