@@ -1,5 +1,3 @@
-#!/usr/bin/wish
-
 ########################################################################
 ##             skins.tcl - aMSN Skins System (0.95B)                  ##
 ##  ----------------------------------------------------------------  ##
@@ -110,11 +108,25 @@ namespace eval ::skin {
 		if { ! [info exists ${location}_names($pixmap_name) ] } {
 			return ""
 		}
+
+		#for better image naming convention
+		switch "$location" {
+				smileys {
+					set naming "emoticonStd_std"
+				}
+				pixmaps {
+					set naming "uiElement_std"
+				}
+				default {
+					set naming $location
+				}
+		}
+
 		#If the loading pixmap is corrupted (like someone stupid trying to change the smileys by himself and is doing something bad), just show a null picture
-		if { [catch {set loaded_${location}($pixmap_name) [image create photo -file [::skin::GetSkinFile ${location} [set ${location}_names($pixmap_name)] \
+		if { [catch {set loaded_${location}($pixmap_name) [image create photo ${naming}_${pixmap_name} -file [::skin::GetSkinFile ${location} [set ${location}_names($pixmap_name)] \
 			"" [set ${location}_fblocation($pixmap_name)]] -format cximage] } res ] } {
 		 	status_log "Error while loading pixmap $res"
-		 	set loaded_${location}($pixmap_name) [image create photo -file [::skin::GetSkinFile pixmaps null \
+		 	set loaded_${location}($pixmap_name) [image create photo ${naming}_${pixmap_name} -file [::skin::GetSkinFile pixmaps null \
 			 -format cximage]]
 		 }
 	
@@ -125,41 +137,122 @@ namespace eval ::skin {
 	################################################################
 	# ::skin::getNoDisplayPicture ([skin_name])
 	# Checks if the image was previously loaded, or we need to load
-	# it. This way, the no_pic will be loaded first time it's used.
-	# It always returns 'no_pic', is always the same name (static).
+	# it. This way, the displaypicture_std_none will be loaded first time it's used.
+	# It always returns 'displaypicture_std_none', is always the same name (static).
 	# Arguments:
 	#  - skin_name => [NOT REQUIRED] Overrides the current skin.
 	proc getNoDisplayPicture { {skin_name ""} } {
 		variable loaded_images
-		if { [info exists loaded_images(no_pic)] } {
-			return no_pic
+		if { [info exists loaded_images(displaypicture_std_none)] } {
+			return displaypicture_std_none
 		}
-		image create photo no_pic -file [::skin::GetSkinFile displaypic nopic.gif $skin_name] -format cximage
-		set loaded_images(no_pic) 1
-		return no_pic
+		image create photo displaypicture_std_none -file [::skin::GetSkinFile displaypic nopic.gif $skin_name] -format cximage
+		set loaded_images(displaypicture_std_none) 1
+		return displaypicture_std_none
 	}
 
-	proc getDisplayPicture { email } {
+	proc getDisplayPicture { email {force 0}} {
 		global HOME
-		if {![catch {image width user_pic_$email}] } {
-			return user_pic_$email
+				
+		set picName displaypicture_std_$email
+		
+		if {[catch {image width $picName}] == 0  && $force == 0} {
+			return $picName
 		}
 
 		set filename [::abook::getContactData $email displaypicfile ""]
-		if { [file readable "[file join $HOME displaypic cache ${filename}].png"] } {
-			catch {image create photo user_pic_$email -file "[file join $HOME displaypic cache ${filename}].png" -format cximage}
+		set file "[file join $HOME displaypic cache ${filename}].png"
+		if { $filename != "" && [file readable "[file join $HOME displaypic cache ${filename}].png"] } {
+			catch {image create photo $picName -file "$file" -format cximage}
 		} else {
-			image create photo user_pic_$email -file [::skin::GetSkinFile displaypic nopic.gif] -format cximage
+			# We do like that to have a better update of the image when switching from no pic to another
+			#image create photo $picName -file [::skin::GetSkinFile displaypic nopic.gif] -format cximage
+			image create photo $picName
+			$picName copy [::skin::getNoDisplayPicture]
+			set file [::skin::GetSkinFile displaypic nopic.gif]
 		}
 		
-		if {[catch {image width user_pic_$email} res] } {
-			#status_log "Error while loading user_pic_$email: $res"
-			return [::skin::getNoDisplayPicture]
+		if {[catch {image width $picName} res]} {
+			#status_log "Error while loading $picName: $res"
+			image create photo $picName
+			$picName copy [::skin::getNoDisplayPicture]
+			set file [::skin::GetSkinFile displaypic nopic.gif]
 		}
 		
-		return user_pic_$email
+		if { [catch {::picture::GetPictureSize $file} cursize] } {
+			#Corrupted image.. might as well delete it and redownload it some other time..
+			status_log "Error opening $file: $cursize\n"
+			catch {file delete $file}
+		} elseif { [::config::getKey autoresizedp] && ![::picture::IsAnimated $file] && $cursize != "96x96"} {
+			::picture::Resize $picName 96 96
+		}
+		if { [catch {image height displaypicture_tny_$email} height] == 0} {
+			#Little DP exists so we have to update it
+			::skin::getLittleDisplayPicture $email $height 1
+		}
+		return $picName
 	}
 
+	proc getLittleDisplayPicture { email height {force 0}} {
+		global HOME
+
+		set picName displaypicture_tny_$email
+		
+		if {[catch {image width $picName}] == 0  && $force == 0} {
+			return $picName
+		}
+
+		set filename [::abook::getContactData $email displaypicfile ""]
+		set file "[file join $HOME displaypic cache ${filename}].png"
+
+		if { $filename != "" && [file readable $file] } {
+			catch {image create photo $picName -file "$file" -format cximage}
+		} else {
+			image create photo $picName
+			$picName copy [::skin::getNoDisplayPicture]
+			set file [::skin::GetSkinFile displaypic nopic.gif]
+		}
+		
+		if {[catch {image width $picName} res]} {
+			#status_log "Error while loading $picName: $res"
+			image create photo $picName
+			$picName copy [::skin::getNoDisplayPicture]
+			set file [::skin::GetSkinFile displaypic nopic.gif]
+		}
+		
+		if { [catch {::picture::GetPictureSize $file} cursize] } {
+			status_log "Error opening $file: $cursize\n"
+			return ""
+		}
+
+		set animated [::picture::IsAnimated $file]
+
+		if { $animated == 0 } {
+			::picture::ResizeWithRatio $picName $height $height
+			return $picName
+		} else {
+			set tmpPic [image create photo [TmpImgName]]  ;#gets destroyed
+			$tmpPic copy $picName
+			image delete $picName
+			image create photo $picName
+			$picName copy $tmpPic
+			image delete $tmpPic
+			::picture::ResizeWithRatio $picName $height $height
+			return $picName
+		}
+		return ""
+	}
+
+	#Convert a display picture from a user to another size
+	proc ConvertDPSize {user width height} {
+		if {[catch {
+			::picture::Resize [getDisplayPicture $user] $width $height
+		} res]} {
+			msg_box $res
+		}
+	}
+
+
 
 	################################################################
 	# ::skin::getColorBar ([skin_name])
@@ -170,28 +263,40 @@ namespace eval ::skin {
 	proc getColorBar { {skin_name ""} } {
 		# Get the contact list width
 		global pgBuddyTop
+		global pgbuddy_colorbar_width
 
-		# Delete old mainbar, and load colorbar
+		set win_width [winfo width [winfo parent $pgBuddyTop]]
+
+		# Avoid deleting/recreating the colorbar every CL refresh!
+		if { [info exists pgbuddy_colorbar_width] && $pgbuddy_colorbar_width == $win_width } {
+			return uiDynamicElement_mainbar
+		}
+
+		set width $win_width
+		# Delete old uiDynamicElement_mainbar, and load colorbar
 		# The colorbar will be loaded as follows:
 		# [first 10 px][11th px repeating to fill][the rest of the colorbar]
-		catch {image delete mainbar}
+		catch {image delete uiDynamicElement_mainbar}
 
 		set barheight [image height [loadPixmap colorbar]]
 		set barwidth [image width [loadPixmap colorbar]]
-		set barendwidth [expr $barwidth - 11]
-		set width [winfo width [winfo parent $pgBuddyTop]]
+		set barendwidth [expr {$barwidth - 11}]
 		if { $width < $barwidth } {
 			set width $barwidth
 		}
-		set barendstart [expr $width - $barendwidth]
+		set barendstart [expr {$width - $barendwidth}]
+
+
 		# Create the color bar copying from the pixmap
-		image create photo mainbar -width $width -height $barheight
-		mainbar blank
-		mainbar copy [loadPixmap colorbar] -from 0 0 10 $barheight
-		mainbar copy [loadPixmap colorbar] -from 10 0 11 $barheight -to 10 0 $barendstart $barheight
-		mainbar copy [loadPixmap colorbar] -from [expr $barwidth - $barendwidth] 0 $barwidth $barheight -to $barendstart 0 $width $barheight
+		image create photo uiDynamicElement_mainbar -width $width -height $barheight
+		uiDynamicElement_mainbar blank
+		uiDynamicElement_mainbar copy [loadPixmap colorbar] -from 0 0 10 $barheight
+		uiDynamicElement_mainbar copy [loadPixmap colorbar] -from 10 0 11 $barheight -to 10 0 $barendstart $barheight
+		uiDynamicElement_mainbar copy [loadPixmap colorbar] -from [expr {$barwidth - $barendwidth}] 0 $barwidth $barheight -to $barendstart 0 $width $barheight
+
+		set pgbuddy_colorbar_width $win_width
 		set ::skin::loaded_images(colorbar) 1
-		return mainbar
+		return uiDynamicElement_mainbar
 	}
 
 
@@ -249,8 +354,8 @@ namespace eval ::skin {
 
 		# Now reload special images that need special treatment
 		variable loaded_images
-		if {[info exists loaded_images(no_pic)]} {
-			unset loaded_images(no_pic)
+		if {[info exists loaded_images(displaypicture_std_none)]} {
+			unset loaded_images(displaypicture_std_none)
 			::skin::getNoDisplayPicture $skin_name
 		}
 		if {[info exists loaded_images(colorbar)]} {
@@ -270,8 +375,8 @@ namespace eval ::skin {
 			#We are not using pixmapscroll on Mac OS X
 			# Reload pixmapscroll's images
 			set psdir [LookForExtFolder $skin_name "pixmapscroll"]
-			if {$psdir != 0} {
-				scrollbar reloadimages $psdir
+			if {$psdir != 0 && [info proc ::pixmapscrollbar::reloadimages] != ""} {
+				::pixmapscrollbar::reloadimages $psdir
 				status_log "skin $skin_name 's scrollbar loaded"
 			}	
 		}
@@ -296,6 +401,8 @@ namespace eval ::skin {
 		sxml::register_routine $skin_id "skin:smileys:size" ::skin::SetEmoticonSize
 		sxml::parse $skin_id
 		sxml::end $skin_id
+
+		catch { unset ::emotions_cleared }
 		
 		# Then reload the real skin
 		set skin_id [sxml::init [::skin::GetSkinFile "" settings.xml $skin_name]]
@@ -391,11 +498,14 @@ namespace eval ::skin {
 		#Get file from ~/.amsn/amsn-extras/skins folder
 		lappend locations [file join $HOME2 amsn-extras skins $skin $type]
 		#Get file from default skin
-		lappend locations [file join [set ::program_dir] skins $defaultskin $type]
-		#Get file from fallback location
-		if { ($fblocation != "") } {
-			lappend locations [file join $fblocation]
-		}
+                if { [::skin::getKey ignoredefaultsmileys] != 1 || $type != "smileys" } {
+                        lappend locations [file join [set ::program_dir] skins $defaultskin $type]
+                        #Get file from fallback location
+                        if { ($fblocation != "") } {
+                                lappend locations [file join $fblocation]
+                        }
+                }
+
 
 		foreach location $locations {
 			foreach extension $ext {
@@ -454,7 +564,7 @@ namespace eval ::skin {
 	#Theses 2 procs are used in ::skin::GetSkinFile
 	proc filenoext { filename } {
 		if {[string last . $filename] != -1 } {
-			return "[string range $filename 0 [expr [string last . $filename] - 1]]"
+			return "[string range $filename 0 [expr {[string last . $filename] - 1}]]"
 		} else {
 			return $filename
 		}
@@ -476,7 +586,7 @@ namespace eval ::skin {
 		upvar $saved_data sdata
 
 		foreach key [array names sdata] {
-			set skin([string range $key [expr [string length $cstack] + 1] [string length $key]]) [string trim $sdata($key)]
+			set skin([string range $key [expr {[string length $cstack] + 1}] [string length $key]]) [string trim $sdata($key)]
 		}
 
 		return 0
@@ -497,7 +607,7 @@ namespace eval ::skin {
 		set skins [concat $skins $skins_in_home $skins_in_home2 $skins_in_extras]
 		set skinlist [list]
 
-		foreach skin $skins {
+		foreach skin [lsort -dictionary $skins] {
 			set dir [file dirname $skin]
 			set desc ""
 
@@ -525,7 +635,7 @@ namespace eval ::skin {
 	proc SetConfigKeys {cstack cdata saved_data cattr saved_attr args} {
 		upvar $saved_data sdata
 		foreach key [array names sdata] {
-			::skin::setKey [string range $key [expr [string length $cstack] + 1] [string length $key]] [string trim $sdata($key)]
+			::skin::setKey [string range $key [expr {[string length $cstack] + 1}] [string length $key]] [string trim $sdata($key)]
 		}
 
 		# This bits are used to override certain keys loaded before with specific values for MacOS X (TkAqua)
@@ -574,7 +684,11 @@ namespace eval ::skinsGUI {
 		}
 
 		toplevel $w
-		wm resizable $w 0 0
+		if {[OnMac]} {
+			wm resizable $w 1 1
+		} else {
+			wm resizable $w 0 0
+		}
 		wm title $w "[trans chooseskin]"
 		wm geometry $w +100+100
 
@@ -608,16 +722,16 @@ namespace eval ::skinsGUI {
 		label $w.status -text ""
 		pack $w.status -side bottom
 
-		image create photo blank -width 1 -height 75
-		label $w.main.left.images.blank -image blank
+#		image create photo blank -width 1 -height 75
+#		label $w.main.left.images.blank -image blank
 
-		image create photo blank2 -width 400 -height 1
-		label $w.main.left.images.blank2 -image blank2
+#		image create photo blank2 -width 400 -height 1
+#		label $w.main.left.images.blank2 -image blank2
 
 		set select -1
 		set idx 0
 
-		label $w.getmore -text "[trans getmoreskins]" -fg #0000FF -cursor hand1
+		label $w.getmore -text "[trans getmoreskins]" -fg #0000FF
 
 		button $w.ok -text "[trans ok]" -command "::skinsGUI::SelectSkinOk $w" 
 		button $w.cancel -text "[trans cancel]" -command "::skinsGUI::SelectSkinCancel $w" 
@@ -707,8 +821,8 @@ namespace eval ::skinsGUI {
 		grid $w.main.left.images.6 -in $w.main.left.images -row 1 -column 6
 		grid $w.main.left.images.7 -in $w.main.left.images -row 1 -column 7
 		grid $w.main.left.images.8 -in $w.main.left.images -row 1 -column 8
-		grid $w.main.left.images.blank -in $w.main.left.images -row 1 -column 10
-		grid $w.main.left.images.blank2 -in $w.main.left.images -row 2 -column 1 -columnspan 8
+#		grid $w.main.left.images.blank -in $w.main.left.images -row 1 -column 10
+#		grid $w.main.left.images.blank2 -in $w.main.left.images -row 2 -column 1 -columnspan 8
 
 		$w.main.left.desc configure -state normal
 		$w.main.left.desc delete 0.0 end
@@ -736,6 +850,17 @@ namespace eval ::skinsGUI {
 			destroy .skin_selector.main.left.images.7
 			destroy .skin_selector.main.left.images.8
 		}
+		catch {
+#TODO the destruction of the images should be binded to the destruction of the widget they're in
+			image delete preview1
+			image delete preview2
+			image delete preview3
+			image delete preview4
+			image delete preview5
+			image delete preview6
+			image delete preview7
+			image delete preview8
+		}
 	}
 
 
@@ -759,6 +884,7 @@ namespace eval ::skinsGUI {
 			unset ::skin::skin_reloaded_needs_reset
 			::skin::reloadSkin $skin
 			destroy $w
+			ClearPreview
 		}
 	}
 
@@ -775,5 +901,6 @@ namespace eval ::skinsGUI {
 		}
 		unset ::skin::skin_reloaded_needs_reset
 		destroy $w
+		ClearPreview
 	}
 }
