@@ -1,5 +1,5 @@
 #
-# $Id: smileys.tcl,v 1.156 2005/10/09 23:03:35 roadkillbunny Exp $
+# $Id: smileys.tcl 7047 2006-07-13 21:58:17Z thaven $
 #
 
 if { $initialize_amsn == 1 } {
@@ -38,7 +38,7 @@ namespace eval ::smiley {
 	# having the correct order in the menu) then we add the elements in the array emotions
 	proc newEmoticon {cstack cdata saved_data cattr saved_attr args} {
 		
-		global emotions emotions_data emotions_names
+		global emotions emotions_data emotions_names emotions_cleared
 		upvar $saved_data sdata
 		
 		#Check if no important fields are missing, or emoticon is disabled
@@ -47,6 +47,13 @@ namespace eval ::smiley {
 		if { ! [info exists sdata(${cstack}:text)] } { return 0 }
 		if { ! [info exists sdata(${cstack}:file)] } { return 0 }
 		if { [info exists sdata(${cstack}:disabled)] && [is_true $sdata(${cstack}:disabled)] } { return 0 }
+
+                if { [::skin::getKey ignoredefaultsmileys] == 1 && ! [info exists emotions_cleared] } {
+                        set emotions_cleared 1
+                        unset emotions_data
+                        unset emotions_names
+                        unset emotions
+                }
 		
 		#Get the smiley info: name, text and file, and other
 		#existing fields, like sound or animated. Store them
@@ -77,8 +84,12 @@ namespace eval ::smiley {
 			#Trim text (remove " ") and ignore empty strings
 			if { $text == "" } { continue }
 			set text [string trim $text \"]
+
 			#some smileys are case sensitive, like the moon!
-			#set text [string tolower $text]
+			if { ![info exists emoticon(casesensitive)] ||
+			     ([info exists emoticon(casesensitive)] && [is_false $emoticon(casesensitive)]) } {
+				set text [string tolower $text]
+			}
 			
 			#Delete any previous ocurrence of this smiley if this existed and skin
 			#propertie was set to "" (loaded when loading default skin first)
@@ -159,8 +170,10 @@ namespace eval ::smiley {
 			if { $field_sort == "_dummy_" } {continue}
 			set emotion($field_sort) [string trim $sdata($field)]
 		}
-		#Create the image now and store it
-		if { [catch { image create photo -file $emotion(file) -format cximage } emotion(image_name) ] } {
+
+
+		#Create the image now and store it   ;# $emotion(name) is unique as we store the emoticon in the array under this name (cfr infra)
+		if { [catch { image create photo emoticonCustom_std_$emotion(text) -file $emotion(file) -format cximage } emotion(image_name) ] } {
 			status_log "Error when creating image for emoticon $emotion(name) : $emotion(image_name)"
 			#Error when creating smiley's image so we don't add it
 			return 0
@@ -299,13 +312,36 @@ namespace eval ::smiley {
 			
 			#Keep searching until no matches
 			set start $textbegin
-			while {[set pos [$tw search -exact $nocase -- $symbol $start $end]] != ""} {
-			
+			while {[set pos [$tw search -exact $nocase -- $symbol $start $end]] != "" } {
+				set start [$tw index ${pos}+1c]
+				set endpos [$tw index $pos+[string length $symbol]c]
+
+				#set skip 0
+				# used to avoid invalid search caused by elided text, such as in ::op
+				if { [string equal -nocase [$tw get $pos $endpos]  $symbol] == 0} {
+					continue
+				#} else {
+				#	for { set i $pos } {$i <= $endpos && !$skip} {set i [$tw index $i+1c]} {
+				#		set tags [$tw tag names $i]
+				#		foreach tag_name $tags {
+				#			set elided [$tw tag cget $tag_name -elide]
+				#			if {$elided != "" && $elided} {
+				#				set skip 1
+				#				break;
+				#			}
+				#		}
+				#	}
+				}
 				
+				#if { $skip } {
+				#	continue
+				#}
+
 				set start [::smiley::SubstSmiley $tw $pos $symbol $image_name $image_file $animated $sound]
 				#If SubstSmiley returns -1, start from beggining.
 				#See why in SubstSmiley. This is a fix
 				if { $start == -1 } { set start $textbegin }
+				
 		
 			}
 			
@@ -367,7 +403,7 @@ namespace eval ::smiley {
 		
 		wm state $w normal
 		set x [expr {$x - 15}]
-		set y [expr {$y + 15 - [winfo height $w]}]
+		set y [expr {$y + 15 - [winfo reqheight $w]}]
 		wm geometry $w +$x+$y
 		#It won't work on Windows without this
 		update idletasks
@@ -447,6 +483,7 @@ namespace eval ::smiley {
 		set llength 1
 
 		#Search for all possible emotions, after they are sorted by symbol length
+		SortSmileys
 		foreach emotion_data [concat $sortedemotions [list [list "\n" "newline"]]] {
 
 			#Symbol is first element
@@ -529,7 +566,7 @@ namespace eval ::smiley {
 					if { $p2 == "__newline__" } {
 						set l [lreplace $l $listpos $listpos [list text $p1] [list "newline"] [list text $p3] ]
 					} else {
-						set l [lreplace $l $listpos $listpos [list text $p1] [list smiley $p2] [list text $p3] ]
+						set l [lreplace $l $listpos $listpos [list text $p1] [list smiley $p2 $symbol] [list text $p3] ]
 					}
 
 					incr llength 2
@@ -560,7 +597,8 @@ namespace eval ::smiley {
 	#Create ONE smiley in the smileys menu
 	proc CreateSmileyInMenu {w cols rows smiw smih emot_num name symbol image file animated} {
 		catch {
-			set resized [image create photo]
+			#number of menu-item in name as I'm usure about the name being unique and the quoting of the symbol
+			set resized [image create photo emoticonAll_tny_${emot_num}_small]
 			$resized copy $image
 			if {[image width $image] > 22 && [image height $image] > 22} {
 				::picture::ResizeWithRatio $resized 22 22
@@ -692,8 +730,8 @@ namespace eval ::smiley {
 
 		toplevel .new_custom
 		
-		set w [LabelFrame:create .new_custom.lfname -text [trans smileconfig] -font splainf]
-		pack $w -anchor n -side top -expand 1 -fill x
+		set w [labelframe .new_custom.lfname -text [trans smileconfig] -font splainf]
+		#pack $w -anchor n -side top -expand 1 -fill x
 		
 		frame .new_custom.1 -class Degt
 		label .new_custom.1.smile -image [::skin::loadPixmap smile]
@@ -738,7 +776,7 @@ namespace eval ::smiley {
 		pack .new_custom.buttons.ok -side right -padx 5
 		pack .new_custom.buttons.cancel -side right -padx 5
 		pack .new_custom.buttons.delete -side left -padx 5
-		pack .new_custom.buttons -side bottom -fill x -expand true -padx 5 -pady 5
+		pack .new_custom.buttons -side bottom -fill x -expand false -padx 5 -pady 5
 
 		pack .new_custom.1.intro -fill both -side left
 
@@ -763,8 +801,8 @@ namespace eval ::smiley {
 		grid $w.casesensitive -row 7 -column 1 -columnspan 2 -padx 2 -pady 2 -sticky w
 		
 		
-		pack .new_custom.1 -expand 1 -fill both -side top -pady 5 -padx 5
-		pack .new_custom.lfname -expand 1 -fill both -side top
+		pack .new_custom.1 -expand false -fill x -side top -pady 5 -padx 5
+		pack .new_custom.lfname -expand 1 -fill both -side top -pady 5 -padx 5
 		
 		UpdateEnabledSoundSmileys
 		
@@ -784,13 +822,15 @@ namespace eval ::smiley {
 	proc NewCustomEmoticonGUI_Delete { name } {
 		global custom_emotions
 		
+		catch {image delete $name}
+
 		unset custom_emotions($name)
 		if { [winfo exists .smile_selector]} {destroy .smile_selector}
 		
 		#After modifying, clear sortedemotions, could need sorting again
 		variable sortedemotions
 		if {[info exists sortedemotions]} { unset sortedemotions }
-		
+
 		destroy .new_custom
 
 	}
@@ -798,7 +838,7 @@ namespace eval ::smiley {
 	proc UpdateEnabledSoundSmileys { } {
 		global new_custom_cfg
 		
-		set w .new_custom.lfname.f.f
+		set w .new_custom.lfname
 		
 		if { $new_custom_cfg(enablesound) == 1 } {
 			$w.sound configure -state normal
@@ -851,7 +891,15 @@ namespace eval ::smiley {
 		set w .new_custom
 
 		if { $name == "" } {
-			set name $new_custom_cfg(name)
+			if { [catch {set name $new_custom_cfg(name)} ] } {
+				#User entered no description
+				msg_box "[trans wrongfields [trans description] [trans triggers] [trans smilefile] ]"
+				return -1
+			} elseif { [info exists custom_emotions($name)] } {
+				#Smiley exists
+				msg_box "[trans smileyexists]"
+				return -1
+			}
 			#set name "[string map { "\[" "\\\[" "\]" "\\\]" } $new_custom_cfg(name)]"
 			set edit 0
 		} else {
@@ -906,18 +954,18 @@ namespace eval ::smiley {
 			} else {
 				set titleid custom_new
 			}
-			image create photo temp_smiley -file [::skin::GetSkinFile smileys "$new_custom_cfg(file)"]
+			image create photo tmp -file [::skin::GetSkinFile smileys "$new_custom_cfg(file)"]
 			
 			set filetail_noext [filenoext [file tail "$new_custom_cfg(file)"]]
 			set destfile [file join $HOME smileys $filetail_noext]
 			
-			if { [image width temp_smiley] > 19 || [image height temp_smiley] > 19 } {
+			if { [image width tmp] > 19 || [image height tmp] > 19 } {
 				#Smiley is static and bigger than 19x19 we ask for resizing
 				if { [::amsn::messageBox "[trans smiletoobig]" yesno question "[trans $titleid ]"] == "yes" } {
 					#The user wants resize it
 					set file [convert_image_plus [::skin::GetSkinFile smileys "$new_custom_cfg(file)"] smileys 19x19]
 				} else {
-					if { [image width temp_smiley] > 50 || [image height temp_smiley] > 50 } {
+					if { [image width tmp] > 50 || [image height tmp] > 50 } {
 						#MSN can't show static smileys which are bigger than 50x50 so we resize it
 						set file [convert_image_plus [::skin::GetSkinFile smileys "$new_custom_cfg(file)"] smileys 50x50]
 					} else {
@@ -977,7 +1025,7 @@ namespace eval ::smiley {
 		
 		
 		
-		set emotion(image_name) [image create photo -file $emotion(file) -format cximage]
+		set emotion(image_name) [image create photo emoticonCustom_std_$emotion(text) -file $emotion(file) -format cximage]
 		set custom_emotions($name) [array get emotion]
 
 		#load_smileys
@@ -993,12 +1041,14 @@ namespace eval ::smiley {
 		save_config
 	}
 	
-	proc addSmileyFromTW { file } {
+	proc addSmileyFromTW { file text } {
+		set text [string map {"\\\\" "\\"} $text]
 		global new_custom_cfg
 		if { [winfo exists .new_custom] } {
 			raise .new_custom
 			return
 		}
+		set new_custom_cfg(text) $text
 		set new_custom_cfg(file) [::skin::GetSkinFile smileys "cache/${file}.png"]
 		newCustomEmoticonGUI
 	}
@@ -1079,61 +1129,77 @@ proc is_true { data } {
 
 
 proc custom_smile_subst { chatid tw {textbegin "0.0"} {end "end"} } {
-    upvar #0 [string map {: _} ${chatid} ]_smileys emotions
+	upvar #0 [string map {: _} ${chatid} ]_smileys emotions
+	
+	set scrolling [::ChatWindow::getScrolling $tw]
+	
+	if { ![info exists emotions] } { return }
 
-    if { ![info exists emotions] } { return }
+	#status_log "Parsing text for [array names emotions] with tw = $tw, textbegin = $textbegin and end = $end\n"
+	#status_log "text to parse : [$tw get $textbegin $end]\n"
 
-    after 250 "custom_smile_subst2 $chatid $tw $textbegin $end"
+	foreach symbol [array names emotions] {
+		set chars [string length $symbol]
+		set file [::MSNP2P::GetFilenameFromMSNOBJ $emotions($symbol)]
+		if { $file == "" } { continue }
+		
+		#status_log "Got file $file for symbol -$symbol-\n" red
+		
+		set start $textbegin
+		
+		# TODO this still needs to be fixed by using [$tw get $start $end] and searching in the text/removing what we found
+		# in the text, because tk 8.4 has a bug with elided text, it's fixed in 8.5, I'll fill a bug report soon and 
+		# add it in the comments...
+		while {[set pos [$tw search -exact -elide -- $symbol $start $end]] != ""} {
+			set start ${pos}+1c
+			set endpos [$tw index $pos+[string length $symbol]c]
+			
+			set skip 0
+			# used to avoid invalid search caused by elided text, such as in ::op
+			if { [string compare [$tw get $pos $endpos]  $symbol] != 0} {
+				continue
+			} else {
+				for { set i $pos } {$i <= $endpos && !$skip } {set i [$tw index $i+1c]} {
+					set tags [$tw tag names $i]
+					foreach tag_name $tags {
+						set elided [$tw tag cget $tag_name -elide]
+						if {$elided != "" && $elided} {
+							set skip 1
+							break
+						}
+					}
+				}
+			}
+			if {$skip} {
+				continue
+			}
 
-} 
+			# Also skip if we are part of a url
+			if { [lsearch -exact [$tw tag names $pos] "dont_replace_smileys"] != -1 } {
+				continue
+			}
 
-proc custom_smile_subst2 { chatid tw textbegin end } { 
-    upvar #0 [string map {: _} ${chatid} ]_smileys emotions
-
-    set scrolling [::ChatWindow::getScrolling $tw]
-
-    if { ![info exists emotions] } { return }
-
-    status_log "Parsing text for [array names emotions] with tw = $tw, textbegin = $textbegin and end = $end\n"
-
-    foreach symbol [array names emotions] {
-	set chars [string length $symbol]
-	set file [::MSNP2P::GetFilenameFromMSNOBJ $emotions($symbol)]
-	if { $file == "" } { continue }
-
-	status_log "Got file $file for symbol -$symbol-\n" red
-
-	set start $textbegin
-	status_log "result $tw search -exact -nocase bb $start $end : [$tw search -exact -nocase bb $start $end]--- $start -- $textbegin\n"
-
-	while {[set pos [$tw search -exact -nocase -- $symbol $start $end]] != ""} {
-	    status_log "Found match at pos : $pos\n" red
-
-	    set posyx [split $pos "."]
-	    set endpos "[lindex $posyx 0].[expr {[lindex $posyx 1] + $chars}]"
-
-
-	    $tw tag configure smiley -elide true
-	    $tw tag add smiley $pos $endpos
-	    
-	    set twTag "custom_smiley_$file"
-	    set copyMenu "${tw}.custom_smiley_$file"
-	    if { ![winfo exists $copyMenu] } {
-		menu $copyMenu -tearoff 0 -type normal
-		$copyMenu add command -label [trans emoticon_steal] -command "::smiley::addSmileyFromTW {$file}"
-		$tw tag bind $twTag <Enter> "$tw configure -cursor hand2"
-		$tw tag bind $twTag <Leave> "$tw configure -cursor xterm"
-		$tw tag bind $twTag <<Button1>> "tk_popup $copyMenu %X %Y"
+			$tw tag configure smiley -elide true
+			$tw tag add smiley $pos $endpos
+			
+			set twTag "emoticonCustom_std_$file"
+			set copyMenu "${tw}.emoticonCustom_std_$file"
+			if { ![winfo exists $copyMenu] } {
+				menu $copyMenu -tearoff 0 -type normal
+				$copyMenu add command -label "[trans emoticon_steal] ($symbol)" -command "::smiley::addSmileyFromTW {$file} {[string map {"\\" "\\\\"} $symbol]}"
+				$tw tag bind $twTag <Enter> "$tw configure -cursor hand2"
+				$tw tag bind $twTag <Leave> "$tw configure -cursor xterm"
+				$tw tag bind $twTag <<Button1>> "tk_popup $copyMenu %X %Y"
+		    }
+			
+			set smileyIdx [$tw image create $endpos -image "emoticonCustom_std_$file" -padx 0 -pady 0]
+			$tw tag add $twTag $smileyIdx
+			$tw tag remove smiley $endpos
+		    
 	    }
-
-	    set smileyIdx [$tw image create $endpos -image "custom_smiley_$file" -padx 0 -pady 0]
-	    $tw tag add $twTag $smileyIdx
-	    $tw tag remove smiley $endpos
-	    
-	}
     }
 
-    unset emotions
+	#unset emotions
     
     if { $scrolling } { ::ChatWindow::Scroll $tw }
 }
@@ -1143,7 +1209,9 @@ proc parse_x_mms_emoticon { data chatid } {
     upvar #0 [string map {: _} ${chatid} ]_smileys smile
 
 
-    if { [::config::getKey getdisppic] != 1 } { return }
+    #Line below changed from != -1 to == 0 because -1 means
+    #"enabled but imagemagick unavailable"
+    if { [::config::getKey getdisppic] == 0 } { return }
 
     set start 0
     while { $start < [string length $data]} {
@@ -1160,7 +1228,8 @@ proc parse_x_mms_emoticon { data chatid } {
     status_log "Got smileys : [array names smile]\n" 
 
 }
-proc process_custom_smileys_SB { txt } {
+
+proc process_custom_smileys_SB { txt {animated 0} } {
 	global custom_emotions
 	
 	set msg ""
@@ -1176,18 +1245,28 @@ proc process_custom_smileys_SB { txt } {
 		}
 		
 		array set emotion $custom_emotions($name)
-		foreach symbol $emotion(text) {
+		foreach symbol [encoding convertto identity $emotion(text)] {
 			set symbol2 [string toupper $symbol]
 		
 			set file $emotion(file)
-			if { ! [ info exists emotion(animated) ] || ! [ is_true $emotion(animated) ] } {
+			if { ($animated && ([ info exists emotion(animated) ] && [ is_true $emotion(animated) ])) ||
+			     (!$animated && (! [ info exists emotion(animated) ] || ! [ is_true $emotion(animated) ]))} {
 				if { [info exists emotion(casesensitive)] && [is_true $emotion(casesensitive)] } {
 					if {  [string first $symbol $txt] != -1 } {
-						set msg "$msg$symbol	[create_msnobj [::config::getKey login] 2 [::skin::GetSkinFile smileys [filenoext $file].png]]	"
+						append msg "$symbol\t[create_msnobj [::config::getKey login] 2 [::skin::GetSkinFile smileys [filenoext $file].png]]\t"
 					}
 				} else {
-					if {  [string first $symbol2 $txt2] != -1 } {
-						set msg "$msg$symbol	[create_msnobj [::config::getKey login] 2 [::skin::GetSkinFile smileys [filenoext $file].png]]	"
+					set msnobj ""
+					set startidx 0
+					while {  [string first $symbol2 $txt2 $startidx] != -1 } {
+						if { $msnobj == "" } {
+							set msnobj [create_msnobj [::config::getKey login] 2 [::skin::GetSkinFile smileys [filenoext $file].png]]
+						}
+						
+						set idx [string first $symbol2 $txt2 $startidx]
+						set startidx [expr {$idx + [string length $symbol2]}]
+						set symbol [string range $txt $idx [expr {$startidx - 1}]]
+						append msg "$symbol\t$msnobj\t"
 					}
 				}
 			}
@@ -1197,40 +1276,8 @@ proc process_custom_smileys_SB { txt } {
 	return $msg
 }
 
+
+
 proc process_custom_animated_smileys_SB { txt } {
-	global custom_emotions
-	
-	set msg ""
-	
-	set txt2 [string toupper $txt]
-
-	#Try to find used smileys in the message	
-	foreach name [array names custom_emotions] {
-	
-		if { ![info exists custom_emotions($name)] } {
-			status_log "process_custom_smileys_SB: Custom smiley $name doesn't exist in custom_emotions array!!\n" red
-			continue
-		}
-		
-		array set emotion $custom_emotions($name)
-		foreach symbol $emotion(text) {
-			set symbol2 [string toupper $symbol]
-		
-			set file $emotion(file)
-			if { [ info exists emotion(animated) ] && [ is_true $emotion(animated) ] } {
-				if { [info exists emotion(casesensitive)] && [is_true $emotion(casesensitive)] } {
-					if {  [string first $symbol $txt] != -1 } {
-						set msg "$msg$symbol	[create_msnobj [::config::getKey login] 2 [::skin::GetSkinFile smileys [filenoext $file].png]]	"
-					}
-				} else {
-					if {  [string first $symbol2 $txt2] != -1 } {
-						set msg "$msg$symbol	[create_msnobj [::config::getKey login] 2 [::skin::GetSkinFile smileys [filenoext $file].png]]	"
-					}
-				}
-			}
-		}
-	}
-	
-	return $msg
+	return [process_custom_smileys_SB $txt 1]
 }
-
