@@ -13,21 +13,36 @@ proc iconify_proc {} {
 	
 	if { [OnWin] } {
 		taskbar_icon_handler WM_LBUTTONDBLCLK 0 0
-		return
-	}
-
-	if { [focus] == "."} {
-		wm iconify .
-		wm state . withdrawn
 	} else {
-		wm deiconify .
-		wm state . normal
-		raise .
-		focus -force .
+		if { [winfo exists .bossmode] } {
+			wm state .bossmode normal
+			raise .bossmode
+			focus -force .bossmode
+			return
+		}
+		if { [focus] == "."} {
+			wm iconify .
+			wm state . withdrawn
+		} else {
+			wm deiconify .
+			wm state . normal
+			raise .
+			focus -force .
+		}
 	}
 	#bind $statusicon <Button-1> deiconify_proc
 }
 
+
+proc isWinicoLoaded {} {
+	foreach lib [info loaded] {
+		if {[lindex $lib 1] == "Winico" } {
+			return 1
+		}
+	}
+	return 0
+}
+
 # Load the needed library (platform dependent) and create the context-menu
 proc trayicon_init {} {
 	global systemtray_exist password iconmenu wintrayicon statusicon
@@ -37,16 +52,20 @@ proc trayicon_init {} {
 		if { $statusicon != 0 } {
 			return
 		}
-		set ext "[file join utils windows winico05.dll]"
-		if { [file exists $ext] != 1 } {
-			msg_box "[trans needwinico2]"
-			close_dock
-			return
-		}
-		if { [catch {load $ext winico}] }	{
-			close_dock
-			return
+		catch {package require Winico}
+		if {![isWinicoLoaded] } {
+			set ext "[file join utils windows winico05.dll]"
+			if { [file exists $ext] != 1 } {
+				msg_box "[trans needwinico2]"
+				close_dock
+				return
+			}
+			if { [catch {load $ext winico}] }	{
+				close_dock
+				return
+			}
 		}
+
 		set systemtray_exist 1
 		set wintrayicon [winico create [::skin::GetSkinFile winicons msn.ico]]
 		#add the icon
@@ -55,7 +74,7 @@ proc trayicon_init {} {
 	} else {
 		if { [catch {package require libtray} res] } {
 			set systemtray_exist 0
-			puts "[trans traynotcompiled] : $res"
+			status_log "[trans traynotcompiled] : $res"
 			close_dock
 			return
 		}
@@ -68,7 +87,7 @@ proc trayicon_init {} {
 	destroy .trayiconwin
 	toplevel .trayiconwin -class Amsn
 	wm overrideredirect .trayiconwin 1
-	wm geometry .trayiconwin "+0+[expr 2 * [winfo screenheight .]]"
+	wm geometry .trayiconwin "+0+[expr {2 * [winfo screenheight .]}]"
 	wm state .trayiconwin withdrawn
 	destroy .trayiconwin.immain
 	set iconmenu .trayiconwin.immain
@@ -89,20 +108,10 @@ proc trayicon_init {} {
 
 	$iconmenu add command -label "[trans offline]" -command iconify_proc
 	$iconmenu add separator
-	if { [string length [::config::getKey login]] > 0 } {
-	     if {$password != ""} {
-	        #$iconmenu add command -label "[trans login] [::config::getKey login]" -command "::MSN::connect" -state normal
-	     } else {
-	     	#$iconmenu add command -label "[trans login] [::config::getKey login]" -command cmsn_draw_login -state normal
-	     }
-	} else {
-	     #$iconmenu add command -label "[trans login]" -command "::MSN::connect" -state disabled
-	}
-	#$iconmenu add command -label "[trans login]..." -command cmsn_draw_login
 
+	$iconmenu add command -label "[trans sendmsg]..." -command [list ::amsn::ShowUserList [trans sendmsg] ::amsn::chatUser] 
 
-
- #     $iconmenu add separator         
+       $iconmenu add separator         
  
        $iconmenu add cascade -label "[trans mystatus]" -menu $iconmenu.imstatus -state disabled        
  
@@ -113,15 +122,19 @@ proc trayicon_init {} {
  #     $iconmenu add command -label "[trans preferences]..." -command Preferences
  
        $iconmenu add separator         
+
+       $iconmenu add command -label "[trans gotoinbox]" -command ::hotmail::hotmail_login
+       $iconmenu add separator
  
  #the login/logout one, defined later on (see below)
        $iconmenu add command
-       $iconmenu add command -label "[trans close]" -command "close_cleanup;exit"
+       $iconmenu add command -label "[trans quit]" -command "exit"
        CreateStatesMenu .my_menu
 
+       statusicon_proc [::MSN::myStatusIs]
+
        ## set icon to current status if added icon while already logged in
        if { [::MSN::myStatusIs] != "FLN" } {
-               statusicon_proc [::MSN::myStatusIs]
                mailicon_proc [::hotmail::unreadMessages]
        }
 
@@ -144,7 +157,7 @@ proc taskbar_icon_handler { msg x y } {
 
 		#workaround for bug with the popup not unposting
 		wm state .trayiconwin normal
-		wm geometry .trayiconwin "+0+[expr 2 * [winfo screenheight .]]"
+		wm geometry .trayiconwin "+0+[expr {2 * [winfo screenheight .]}]"
 		focus -force .trayiconwin
 
 		tk_popup $iconmenu [expr "$x + 85"] [expr "$y - 11"] [$iconmenu index end]
@@ -178,7 +191,7 @@ proc taskbar_icon_handler { msg x y } {
 }
 
 proc trayicon_callback {imgSrc imgDst width height} {
-	$imgDst copy $imgSrc
+	$imgDst copy $imgSrc -compositingrule set
 	if { [image width $imgSrc] > $width || [image height $imgSrc] > $height} {
 		::picture::ResizeWithRatio $imgDst $width $height
 	}
@@ -216,20 +229,23 @@ proc statusicon_proc {status} {
 	} elseif {$systemtray_exist == 1 && $statusicon != 0 && ( [UnixDock] || [WinDock] ) && $status != "REMOVE"} {
 		if { $status != "" } {
 			if { $status == "FLN" } {
-#Status submenu
+				# Send message 
 				$iconmenu entryconfigure 2 -state disabled
-#Change nick
+				#Status submenu
 				$iconmenu entryconfigure 4 -state disabled
+				#Change nick
+				$iconmenu entryconfigure 6 -state disabled
 
-#Login/Logout
-				$iconmenu entryconfigure 7 -label "[trans login]" -command "::MSN::connect" -state normal
+				#Login/Logout
+				$iconmenu entryconfigure 11 -label "[trans login]" -command "::MSN::connect" -state normal
 
 			} else {
 
 				$iconmenu entryconfigure 2 -state normal
 				$iconmenu entryconfigure 4 -state normal
+				$iconmenu entryconfigure 6 -state normal
 
-				$iconmenu entryconfigure 7 -label "[trans logout]" -command "::MSN::logout"
+				$iconmenu entryconfigure 11 -label "[trans logout]" -command "::MSN::logout"
 
 				
 			}
@@ -300,8 +316,9 @@ proc statusicon_proc {status} {
 					set trayicon [winico create [::skin::GetSkinFile winicons hidden.ico]]
 				}
 			  }
-			  "BOSS" {   #for bossmode, only for win at the moment
-				#set pixmap "[::skin::GetSkinFile pixmaps doffline.xpm]"
+			  "BOSS" {   
+				#for bossmode
+				set pixmap "[::skin::GetSkinFile pixmaps dboss.png]"
 				set tooltip "[trans pass]"
 				if { [WinDock] } {
 					set trayicon [winico create [::skin::GetSkinFile winicons bossmode.ico]]
@@ -351,7 +368,7 @@ proc taskbar_mail_icon_handler { msg x y } {
 	}
 
 	if { $msg=="WM_LBUTTONUP" } {
-		::hotmail::hotmail_login [::config::getKey login] $password
+		::hotmail::hotmail_login 
 	}
 }
 
@@ -359,6 +376,11 @@ proc mailicon_proc {num} {
 	# Workaround for bug in the traydock-plugin - statusicon added - BEGIN
 	global systemtray_exist mailicon statusicon password winmailicon tcl_platform mailtrayicon defaultbackground
 	# Workaround for bug in the traydock-plugin - statusicon added - END
+
+	if { [::config::getKey showmailicon] == 0 } {
+		return
+	}
+
 	set icon .mi
 	if {$systemtray_exist == 1 && $mailicon == 0 && ([UnixDock] || [WinDock])  && $num >0} {
 		set pixmap "[::skin::GetSkinFile pixmaps unread_tray.gif]"
@@ -379,7 +401,7 @@ proc mailicon_proc {num} {
 			image create photo mailtrayiconres
 			set mailicon [newti $icon -tooltip offline -pixmap mailtrayiconres -command "::trayicon_callback mailtrayicon mailtrayiconres"]
 
-			bind $icon <Button-1> [list ::hotmail::hotmail_login [::config::getKey login] $password]
+			bind $icon <Button-1> "::hotmail::hotmail_login"
 		}
 
 	} elseif {$systemtray_exist == 1 && $mailicon != 0 && $num == 0} {
@@ -417,7 +439,7 @@ proc remove_icon {icon} {
 
 proc restart_tray { } {
 
-    puts "RESTARTING the traydock"
+    status_log "RESTARTING the traydock"
 
     statusicon_proc "REMOVE"
     statusicon_proc [::MSN::myStatusIs]
@@ -446,31 +468,38 @@ proc restart_tray { } {
 proc loadTrayLib {} {
 	
 	if { [OnWin] } {
+
+		# First try with the package winico (for the 0.6 version)
+		catch {package require Winico}
+		if {[isWinicoLoaded] } {
+			return 1
+		}
+
 		#set file name of the lib
 		set ext "[file join utils windows winico05.dll]"
 
 		#if the lib is not available, print an error on the console and return
 		if { ![file exists $ext] } {
-			puts "[trans needwinico2]"
+			status_log "[trans needwinico2]"
 			return 0
 		}
 
 		#if there is a problem loading the lib, print the error on the console and return
 		if { [catch {load $ext winico} errormsg] } {
-			puts "$errormsg"
+			status_log "$errormsg"
 			return 0
 		}
 
 	} elseif { [OnUnix] } {
 		#if there is a problem loading the lib, print the error on the console and return
 		if { [catch {package require libtray} errormsg] } {
-			puts "[trans traynotcompiled] : $errormsg"
+			status_log "[trans traynotcompiled] : $errormsg"
 			return 0
 		}
 	
 		#check if a systemtray is available, ifnot, set the "lib loaded" state off so no icons will be made
 		if { ![systemtray_exist] } {
-			puts "[trans nosystemtray]"
+			status_log "[trans nosystemtray]"
 			return 0
 		}
 		
@@ -511,7 +540,7 @@ proc addTrayIcon {name xiconpath winiconpath {tooltip ""} {winactionhandler "noh
 		#X11/Freedesktop (linux) specific code
 		} elseif { [OnUnix] && $xiconpath != ""} {
 			if { [winfo exists .$name] } {
-				puts "trayicon.tcl: won't add icon $name as it already exists"
+				status_log "trayicon.tcl: won't add icon $name as it already exists"
 			} else {
 				if { [loadTrayLib] } {
 					#add the icon     !! name => .name
@@ -527,7 +556,7 @@ proc addTrayIcon {name xiconpath winiconpath {tooltip ""} {winactionhandler "noh
 			}
 			return 1
 		} else {
-			puts "Error creating trayicon."
+			status_log "Error creating trayicon."
 			return 0
 		}
 
@@ -552,10 +581,10 @@ proc rmTrayIcon {name} {
 		#X11/Freedesktop (linux) specific code
 		} elseif { [OnUnix] } {
 			if { [catch {removeti .$name} errormsg] } {
-				puts "$errormsg\n"
+				status_log "$errormsg\n"
 			}
 			if { [catch {destroy .$name} errormsg] } {
-				puts "$errormsg\n"
+				status_log "$errormsg\n"
 			}
 		}
 	}
@@ -599,5 +628,5 @@ proc trayIcon_Configure {imgSrc imgDst width height} {
 }
 #Windows only aid proc:
 proc nohandler {win x y} {
-	puts "Err: No icon handler given"
+	status_log "Err: No icon handler given"
 }
